# -*- coding: utf-8 -*-
"""The VirusTotal analysis plugin CLI arguments helper."""

from __future__ import unicode_literals

from plaso.lib import errors
from plaso.cli.helpers import interface
from plaso.cli.helpers import manager
from plaso.analysis import virustotal


class VirusTotalAnalysisArgumentsHelper(interface.ArgumentsHelper):
  """VirusTotal analysis plugin CLI arguments helper."""

  # pylint: disable=arguments-differ

  NAME = 'virustotal_analysis'
  CATEGORY = 'analysis'
  DESCRIPTION = 'Argument helper for the VirusTotal analysis plugin.'

  _DEFAULT_HASH = 'sha256'
  _DEFAULT_RATE_LIMIT = True

  @classmethod
  def AddArguments(cls, argument_group):
    """Adds command line arguments the helper supports to an argument group.

    This function takes an argument parser or an argument group object and adds
    to it all the command line arguments this helper supports.

    Args:
      argument_group (argparse._ArgumentGroup|argparse.ArgumentParser):
          argparse group.
    """
    argument_group.add_argument(
        '--virustotal-api-key', '--virustotal_api_key',
        dest='virustotal_api_key', type=str, action='store', default=None,
        metavar='API_KEY', help=(
            'Specify the API key for use with VirusTotal.'))

    argument_group.add_argument(
        '--virustotal-free-rate-limit', '--virustotal_free_rate_limit',
        dest='virustotal_free_rate_limit',
        action='store_false', default=cls._DEFAULT_RATE_LIMIT, help=(
            'Limit Virustotal requests to the default free API key rate of '
            '4 requests per minute. Set this to false if you have an key '
            'for the private API.'))

    argument_group.add_argument(
        '--virustotal-hash', '--virustotal_hash', dest='virustotal_hash',
        type=str, action='store', choices=['md5', 'sha1', 'sha256'],
        default=cls._DEFAULT_HASH, metavar='HASH', help=(
            'Type of hash to query VirusTotal, the default is: {0:s}'.format(
                cls._DEFAULT_HASH)))

  @classmethod
  def ParseOptions(cls, options, analysis_plugin):
    """Parses and validates options.

    Args:
      options (argparse.Namespace): parser options.
      analysis_plugin (VirusTotalAnalysisPlugin): analysis plugin to configure.

    Raises:
      BadConfigObject: when the output module object is of the wrong type.
      BadConfigOption: when a configuration parameter fails validation or
          when unable to connect to VirusTotal.
    """
    if not isinstance(analysis_plugin, virustotal.VirusTotalAnalysisPlugin):
      raise errors.BadConfigObject(
          'Analysis plugin is not an instance of VirusTotalAnalysisPlugin')

    api_key = cls._ParseStringOption(options, 'virustotal_api_key')
    if not api_key:
      raise errors.BadConfigOption(
          'VirusTotal API key not specified. Try again with '
          '--virustotal-api-key.')

    analysis_plugin.SetAPIKey(api_key)

    enable_rate_limit = getattr(
        options, 'virustotal_free_rate_limit', cls._DEFAULT_RATE_LIMIT)
    if enable_rate_limit:
      analysis_plugin.EnableFreeAPIKeyRateLimit()

    lookup_hash = cls._ParseStringOption(
        options, 'virustotal_hash', default_value=cls._DEFAULT_HASH)
    analysis_plugin.SetLookupHash(lookup_hash)

    if not analysis_plugin.TestConnection():
      raise errors.BadConfigOption('Unable to connect to VirusTotal')


manager.ArgumentHelperManager.RegisterHelper(VirusTotalAnalysisArgumentsHelper)



# -*- coding: utf-8 -*-
"""The psteal CLI tool."""

from __future__ import unicode_literals

import argparse
import collections
import datetime
import logging
import os
import sys
import textwrap

from dfvfs.lib import definitions as dfvfs_definitions
from dfvfs.resolver import context as dfvfs_context

# The following import makes sure the filters are registered.
from plaso import filters  # pylint: disable=unused-import

# The following import makes sure the output modules are registered.
from plaso import output  # pylint: disable=unused-import

from plaso.cli import status_view
from plaso.cli import storage_media_tool
from plaso.cli import tool_options
from plaso.cli import tools
from plaso.cli import views
from plaso.cli.helpers import manager as helpers_manager
from plaso.engine import configurations
from plaso.engine import engine
from plaso.engine import knowledge_base
from plaso.engine import single_process as single_process_engine
from plaso.formatters import mediator as formatters_mediator
from plaso.frontend import utils as frontend_utils
from plaso.lib import errors
from plaso.multi_processing import psort
from plaso.multi_processing import task_engine as multi_process_engine
from plaso.output import interface as output_interface
from plaso.output import manager as output_manager
from plaso.output import mediator as output_mediator
from plaso.parsers import manager as parsers_manager
from plaso.storage import zip_file as storage_zip_file


class PstealTool(
    storage_media_tool.StorageMediaTool,
    tool_options.AnalysisPluginOptions,
    tool_options.HashersOptions,
    tool_options.OutputModuleOptions,
    tool_options.ParsersOptions,
    tool_options.StorageFileOptions):
  """Psteal CLI tool.

  Psteal extract events from the provided source and stores them in an
  intermediate storage file. After extraction an output log file is created.
  This mimics the behaviour of the log2timeline.pl.
  The tool currently doesn't support any of the log2timeline or psort tools'
  flags.

  Attributes:
    dependencies_check (bool): True if the availability and versions of
        dependencies should be checked.
    list_analysis_plugins (bool): True if information about the analysis
        plugins should be shown.
    list_hashers (bool): True if the hashers should be listed.
    list_language_identifiers (bool): True if information about the language
        identifiers should be shown.
    list_parsers_and_plugins (bool): True if the parsers and plugins should
        be listed.
    list_output_modules (bool): True if information about the output modules
        should be shown.
  """

  NAME = 'psteal'

  # TODO: is textwrap.dedent or the join really needed here?
  DESCRIPTION = textwrap.dedent('\n'.join([
      '',
      ('psteal is a command line tool to extract events from individual '),
      'files, recursing a directory (e.g. mount point) or storage media ',
      'image or device. The output events will be stored in a storage file.',
      'This tool will then read the output and process the events into a CSV ',
      'file.',
      '',
      'More information can be gathered from here:',
      '    https://github.com/log2timeline/plaso/wiki/Using-log2timeline',
      '']))

  EPILOG = textwrap.dedent('\n'.join([
      '',
      'Example usage:',
      '',
      'Run the tool against a storage media image (full kitchen sink)',
      '    psteal.py --source ímynd.dd -w imynd.timeline.txt',
      '',
      'And that is how you build a timeline using psteal...',
      '']))

  # The window status-view mode has an annoying flicker on Windows,
  # hence we default to linear status-view mode instead.
  if sys.platform.startswith('win'):
    _DEFAULT_STATUS_VIEW_MODE = status_view.StatusView.MODE_LINEAR
  else:
    _DEFAULT_STATUS_VIEW_MODE = status_view.StatusView.MODE_WINDOW

  _SOURCE_TYPES_TO_PREPROCESS = frozenset([
      dfvfs_definitions.SOURCE_TYPE_DIRECTORY,
      dfvfs_definitions.SOURCE_TYPE_STORAGE_MEDIA_DEVICE,
      dfvfs_definitions.SOURCE_TYPE_STORAGE_MEDIA_IMAGE])

  def __init__(self, input_reader=None, output_writer=None):
    """Initializes the CLI tool object.

    Args:
      input_reader (Optional[InputReader]): input reader, where None indicates
          that the stdin input reader should be used.
      output_writer (Optional[OutputWriter]): output writer, where None
          indicates that the stdout output writer should be used.
    """
    super(PstealTool, self).__init__(
        input_reader=input_reader, output_writer=output_writer)
    self._analysis_plugins = None
    self._artifacts_registry = None
    self._command_line_arguments = None
    self._deduplicate_events = True
    self._enable_sigsegv_handler = False
    self._force_preprocessing = False
    self._knowledge_base = knowledge_base.KnowledgeBase()
    self._number_of_analysis_reports = 0
    self._number_of_extraction_workers = 0
    self._parser_filter_expression = None
    self._parsers_manager = parsers_manager.ParsersManager
    self._preferred_language = 'en-US'
    self._preferred_year = None
    self._resolver_context = dfvfs_context.Context()
    self._single_process_mode = False
    self._status_view_mode = self._DEFAULT_STATUS_VIEW_MODE
    self._status_view = status_view.StatusView(self._output_writer, self.NAME)
    self._storage_file_path = None
    self._time_slice = None
    self._use_time_slicer = False
    self._use_zeromq = True
    self._yara_rules_string = None

    self.list_analysis_plugins = False
    self.list_hashers = False
    self.list_language_identifiers = False
    self.list_parsers_and_plugins = False
    self.list_output_modules = False

  def _CreateProcessingConfiguration(self):
    """Creates a processing configuration.

    Returns:
      ProcessingConfiguration: processing configuration.
    """
    # TODO: pass preferred_encoding.
    configuration = configurations.ProcessingConfiguration()
    configuration.credentials = self._credential_configurations
    configuration.debug_output = self._debug_mode
    configuration.extraction.hasher_names_string = self._hasher_names_string
    configuration.extraction.yara_rules_string = self._yara_rules_string
    configuration.filter_file = self._filter_file
    configuration.parser_filter_expression = self._parser_filter_expression
    configuration.preferred_year = self._preferred_year

    return configuration

  def _GenerateStorageFileName(self):
    """Generates a name for the storage file.

    The result use a timestamp and the basename of the source path.

    Raises:
      BadConfigOption: raised if the source path is not set.
    """
    if not self._source_path:
      raise errors.BadConfigOption('Please define a source (--source).')

    timestamp = datetime.datetime.now()
    datetime_string = timestamp.strftime('%Y%m%dT%H%M%S')

    source_path = os.path.abspath(self._source_path)
    source_name = os.path.basename(source_path)

    if source_path.endswith(os.path.sep):
      source_path = os.path.dirname(source_path)

    source_name = os.path.basename(source_path)

    if not source_name or source_name in ('/', '\\'):
      # The user passed the filesystem's root as source
      source_name = 'ROOT'

    return '{0:s}-{1:s}.plaso'.format(datetime_string, source_name)

  def _PreprocessSources(self, extraction_engine):
    """Preprocesses the sources.

    Args:
      extraction_engine (BaseEngine): extraction engine to preprocess
          the sources.
    """
    logging.debug('Starting preprocessing.')

    try:
      extraction_engine.PreprocessSources(
          self._artifacts_registry, self._source_path_specs,
          resolver_context=self._resolver_context)

    except IOError as exception:
      logging.error('Unable to preprocess with error: {0:s}'.format(exception))

    logging.debug('Preprocessing done.')

  def _ParseOutputModuleOptions(self, options):
    """Parses the output module options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    preferred_time_zone = self._preferred_time_zone or 'UTC'

    formatter_mediator = formatters_mediator.FormatterMediator(
        data_location=self._data_location)

    try:
      formatter_mediator.SetPreferredLanguageIdentifier(
          self._preferred_language)
    except (KeyError, TypeError) as exception:
      raise RuntimeError(exception)

    output_mediator_object = output_mediator.OutputMediator(
        self._knowledge_base, formatter_mediator,
        preferred_encoding=self.preferred_encoding)
    output_mediator_object.SetTimezone(preferred_time_zone)

    try:
      self._output_module = output_manager.OutputManager.NewOutputModule(
          self._output_format, output_mediator_object)

    except IOError as exception:
      raise RuntimeError(
          'Unable to create output module with error: {0:s}'.format(
              exception))

    if not self._output_module:
      raise RuntimeError('Missing output module.')

    if isinstance(self._output_module, output_interface.LinearOutputModule):
      if not self._output_filename:
        raise errors.BadConfigOption((
            'Output format: {0:s} requires an output file').format(
                self._output_format))

      if os.path.exists(self._output_filename):
        raise errors.BadConfigOption(
            'Output file already exists: {0:s}.'.format(self._output_filename))

      output_file_object = open(self._output_filename, 'wb')
      output_writer = tools.FileObjectOutputWriter(output_file_object)

      self._output_module.SetOutputWriter(output_writer)

    helpers_manager.ArgumentHelperManager.ParseOptions(
        options, self._output_module)

  def _PrintAnalysisReportsDetails(self, storage, number_of_analysis_reports):
    """Prints the details of the analysis reports.

    Args:
      storage (BaseStorage): storage writer.
      number_of_analysis_reports (int): number of analysis reports.
    """
    for index, analysis_report in enumerate(storage.GetAnalysisReports()):
      if index + 1 <= number_of_analysis_reports:
        continue

      title = 'Analysis report: {0:d}'.format(index)
      table_view = views.ViewsFactory.GetTableView(
          self._views_format_type, title=title)

      table_view.AddRow(['String', analysis_report.GetString()])

      table_view.Write(self._output_writer)

  def AnalyzeEvents(self):
    """Analyzes events from a plaso storage file and generate a report.

    Raises:
      BadConfigOption: when a configuration parameter fails validation.
      RuntimeError: if a non-recoverable situation is encountered.
    """
    session = engine.BaseEngine.CreateSession(
        command_line_arguments=self._command_line_arguments,
        preferred_encoding=self.preferred_encoding)

    storage_reader = storage_zip_file.ZIPStorageFileReader(
        self._storage_file_path)
    self._number_of_analysis_reports = (
        storage_reader.GetNumberOfAnalysisReports())
    storage_reader.Close()

    counter = collections.Counter()
    if self._output_format != 'null':
      self._status_view.SetMode(self._status_view_mode)
      self._status_view.SetStorageFileInformation(self._storage_file_path)

      status_update_callback = (
          self._status_view.GetAnalysisStatusUpdateCallback())

      storage_reader = storage_zip_file.ZIPStorageFileReader(
          self._storage_file_path)

      # TODO: add single processing support.
      analysis_engine = psort.PsortMultiProcessEngine(
          use_zeromq=self._use_zeromq)

      # TODO: pass configuration object.
      events_counter = analysis_engine.ExportEvents(
          self._knowledge_base, storage_reader, self._output_module,
          deduplicate_events=self._deduplicate_events,
          status_update_callback=status_update_callback,
          time_slice=self._time_slice, use_time_slicer=self._use_time_slicer)

      counter += events_counter

    for item, value in iter(session.analysis_reports_counter.items()):
      counter[item] = value

    if self._quiet_mode:
      return

    self._output_writer.Write('Processing completed.\n')

    table_view = views.ViewsFactory.GetTableView(
        self._views_format_type, title='Counter')
    for element, count in counter.most_common():
      if not element:
        element = 'N/A'
      table_view.AddRow([element, count])
    table_view.Write(self._output_writer)

    storage_reader = storage_zip_file.ZIPStorageFileReader(
        self._storage_file_path)
    self._PrintAnalysisReportsDetails(
        storage_reader, self._number_of_analysis_reports)

    self._output_writer.Write('Storage file is {0:s}\n'.format(
        self._storage_file_path))

  def ExtractEventsFromSources(self):
    """Processes the sources and extract events.

    This is a stripped down copy of tools/log2timeline.py that doesn't support
    the full set of flags. The defaults for these are hard coded in the
    constructor of this class.

    Raises:
      SourceScannerError: if the source scanner could not find a supported
          file system.
      UserAbort: if the user initiated an abort.
    """
    self._CheckStorageFile(self._storage_file_path)

    scan_context = self.ScanSource()
    source_type = scan_context.source_type

    self._status_view.SetMode(self._status_view_mode)
    self._status_view.SetSourceInformation(
        self._source_path, source_type, filter_file=self._filter_file)

    status_update_callback = (
        self._status_view.GetExtractionStatusUpdateCallback())

    self._output_writer.Write('\n')
    self._status_view.PrintExtractionStatusHeader(None)
    self._output_writer.Write('Processing started.\n')

    session = engine.BaseEngine.CreateSession(
        command_line_arguments=self._command_line_arguments,
        filter_file=self._filter_file,
        preferred_encoding=self.preferred_encoding,
        preferred_time_zone=self._preferred_time_zone,
        preferred_year=self._preferred_year)

    storage_writer = storage_zip_file.ZIPStorageFileWriter(
        session, self._storage_file_path)

    configuration = self._CreateProcessingConfiguration()

    single_process_mode = self._single_process_mode
    if source_type == dfvfs_definitions.SOURCE_TYPE_FILE:
      # No need to multi process a single file source.
      single_process_mode = True

    if single_process_mode:
      extraction_engine = single_process_engine.SingleProcessEngine()
    else:
      extraction_engine = multi_process_engine.TaskMultiProcessEngine(
          use_zeromq=self._use_zeromq)

    # If the source is a directory or a storage media image
    # run pre-processing.
    if (self._force_preprocessing or
        source_type in self._SOURCE_TYPES_TO_PREPROCESS):
      self._PreprocessSources(extraction_engine)

    if not configuration.parser_filter_expression:
      operating_system = extraction_engine.knowledge_base.GetValue(
          'operating_system')
      operating_system_product = extraction_engine.knowledge_base.GetValue(
          'operating_system_product')
      operating_system_version = extraction_engine.knowledge_base.GetValue(
          'operating_system_version')
      parser_filter_expression = (
          self._parsers_manager.GetPresetForOperatingSystem(
              operating_system, operating_system_product,
              operating_system_version))

      if parser_filter_expression:
        logging.info('Parser filter expression changed to: {0:s}'.format(
            parser_filter_expression))

      configuration.parser_filter_expression = parser_filter_expression
      session.enabled_parser_names = list(
          self._parsers_manager.GetParserAndPluginNames(
              parser_filter_expression=configuration.parser_filter_expression))
      session.parser_filter_expression = configuration.parser_filter_expression

    # Note session.preferred_time_zone will default to UTC but
    # self._preferred_time_zone is None when not set.
    if self._preferred_time_zone:
      try:
        extraction_engine.knowledge_base.SetTimeZone(self._preferred_time_zone)
      except ValueError:
        # pylint: disable=protected-access
        logging.warning(
            'Unsupported time zone: {0:s}, defaulting to {1:s}'.format(
                self._preferred_time_zone,
                extraction_engine.knowledge_base._time_zone.zone))

    filter_find_specs = None
    if configuration.filter_file:
      environment_variables = (
          extraction_engine.knowledge_base.GetEnvironmentVariables())
      filter_find_specs = frontend_utils.BuildFindSpecsFromFile(
          configuration.filter_file,
          environment_variables=environment_variables)

    processing_status = None
    if single_process_mode:
      logging.debug('Starting extraction in single process mode.')

      processing_status = extraction_engine.ProcessSources(
          self._source_path_specs, storage_writer, self._resolver_context,
          configuration, filter_find_specs=filter_find_specs,
          status_update_callback=status_update_callback)

    else:
      logging.debug('Starting extraction in multi process mode.')

      processing_status = extraction_engine.ProcessSources(
          session.identifier, self._source_path_specs, storage_writer,
          configuration,
          enable_sigsegv_handler=self._enable_sigsegv_handler,
          filter_find_specs=filter_find_specs,
          number_of_worker_processes=self._number_of_extraction_workers,
          status_update_callback=status_update_callback)

    self._status_view.PrintExtractionSummary(processing_status)

  def ParseArguments(self):
    """Parses the command line arguments.

    Returns:
      bool: True if the arguments were successfully parsed.
    """
    argument_parser = argparse.ArgumentParser(
        description=self.DESCRIPTION, epilog=self.EPILOG, add_help=False,
        formatter_class=argparse.RawDescriptionHelpFormatter)

    self.AddBasicOptions(argument_parser)

    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        argument_parser, names=['storage_file'])

    extraction_group = argument_parser.add_argument_group(
        'Extraction Arguments')

    argument_helper_names = [
        'artifact_definitions', 'extraction', 'filter_file', 'hashers',
        'parsers', 'yara_rules']
    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        extraction_group, names=argument_helper_names)

    self.AddCredentialOptions(extraction_group)

    info_group = argument_parser.add_argument_group('Informational Arguments')

    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        info_group, names=['status_view'])

    input_group = argument_parser.add_argument_group('Input Arguments')
    input_group.add_argument(
        '--source', dest='source', action='store',
        type=str, help='The source to process')

    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        input_group, names=['data_location'])

    output_group = argument_parser.add_argument_group('Output Arguments')

    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        output_group, names=['language'])

    self.AddTimeZoneOption(output_group)

    output_format_group = argument_parser.add_argument_group(
        'Output Format Arguments')

    helpers_manager.ArgumentHelperManager.AddCommandLineArguments(
        output_format_group, names=['output_modules'])

    try:
      options = argument_parser.parse_args()
    except UnicodeEncodeError:
      # If we get here we are attempting to print help in a non-Unicode
      # terminal.
      self._output_writer.Write('\n')
      self._output_writer.Write(argument_parser.format_help())
      return False

    try:
      self.ParseOptions(options)
    except errors.BadConfigOption as exception:
      self._output_writer.Write('ERROR: {0!s}\n'.format(exception))
      self._output_writer.Write('\n')
      self._output_writer.Write(argument_parser.format_usage())
      return False

    return True

  def ParseOptions(self, options):
    """Parses tool specific options.

    Args:
      options (argparse.Namespace): command line arguments.

    Raises:
      BadConfigOption: if the options are invalid.
    """
    # The extraction options are dependent on the data location.
    helpers_manager.ArgumentHelperManager.ParseOptions(
        options, self, names=['data_location'])

    # Check the list options first otherwise required options will raise.

    # The output modules options are dependent on the preferred language
    # and preferred time zone options.
    self._ParseTimezoneOption(options)

    argument_helper_names = [
        'analysis_plugins', 'hashers', 'language', 'output_modules',
        'parsers']
    helpers_manager.ArgumentHelperManager.ParseOptions(
        options, self, names=argument_helper_names)

    self.list_hashers = self._hasher_names_string == 'list'
    self.list_language_identifiers = self._preferred_language == 'list'
    self.list_output_modules = self._output_format == 'list'
    self.list_parsers_and_plugins = self._parser_filter_expression == 'list'

    if (self.list_analysis_plugins or self.list_hashers or
        self.list_language_identifiers or self.list_output_modules or
        self.list_parsers_and_plugins or self.list_timezones):
      return

    self._ParseInformationalOptions(options)

    argument_helper_names = [
        'artifact_definitions', 'extraction', 'status_view', 'storage_file',
        'yara_rules']
    helpers_manager.ArgumentHelperManager.ParseOptions(
        options, self, names=argument_helper_names)

    self._ParseLogFileOptions(options)

    self._ParseStorageMediaOptions(options)

    # These arguments are parsed from argparse.Namespace, so we can make
    # tests consistents with the log2timeline/psort ones.
    self._single_process_mode = getattr(options, 'single_process', False)

    if not self._storage_file_path:
      self._storage_file_path = self._GenerateStorageFileName()

    self._analysis_plugins = self._CreateAnalysisPlugins(options)
    self._output_module = self._CreateOutputModule(options)



# -*- coding: utf-8 -*-
"""The Android WebViewCache database event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


# TODO: move to android_webview.py.
class AndroidWebViewCacheFormatter(interface.ConditionalEventFormatter):
  """Formatter for Android WebViewCache event data."""

  DATA_TYPE = u'android:webviewcache'

  FORMAT_STRING_PIECES = [
      u'URL: {url}',
      u'Content Length: {content_length}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{url}']

  SOURCE_LONG = u'Android WebViewCache'
  SOURCE_SHORT = u'WebViewCache'


manager.FormattersManager.RegisterFormatter(AndroidWebViewCacheFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the OpenXML event formatter."""

import unittest

from plaso.formatters import oxml

from tests.formatters import test_lib


class OpenXMLParserFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the OXML event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = oxml.OpenXMLParserFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = oxml.OpenXMLParserFormatter()

    expected_attribute_names = [
        u'creating_app',
        u'app_version',
        u'title',
        u'subject',
        u'last_saved_by',
        u'author',
        u'total_edit_time',
        u'keywords',
        u'comments',
        u'revision_number',
        u'template',
        u'number_of_pages',
        u'number_of_words',
        u'number_of_characters',
        u'number_of_characters_with_spaces',
        u'number_of_lines',
        u'company',
        u'manager',
        u'shared',
        u'security',
        u'hyperlinks_changed',
        u'links_up_to_date',
        u'scale_crop',
        u'dig_sig',
        u'slides',
        u'hidden_slides',
        u'presentation_format',
        u'mm_clips',
        u'notes']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the date filters CLI arguments helper."""

import argparse
import unittest

from plaso.cli import tools
from plaso.cli.helpers import date_filters
from plaso.filters import file_entry as file_entry_filters
from plaso.lib import errors

from tests.cli import test_lib as cli_test_lib


class DateFiltersArgumentsHelperTest(cli_test_lib.CLIToolTestCase):
  """Tests for the date filters CLI arguments helper."""

  # pylint: disable=protected-access

  _EXPECTED_OUTPUT = u'\n'.join([
      u'usage: cli_helper.py [--date-filter TYPE_START_END]',
      u'',
      u'Test argument parser.',
      u'',
      u'optional arguments:',
      u'  --date-filter TYPE_START_END, --date_filter TYPE_START_END',
      (u'                        Filter based on file entry date and time '
       u'ranges. This'),
      u'                        parameter is formatted as',
      (u'                        "TIME_VALUE,START_DATE_TIME,END_DATE_TIME" '
       u'where'),
      (u'                        TIME_VALUE defines which file entry '
       u'timestamp the'),
      (u'                        filter applies to e.g. atime, ctime, '
       u'crtime, bkup,'),
      u'                        etc. START_DATE_TIME and END_DATE_TIME define',
      (u'                        respectively the start and end of the date '
       u'time range.'),
      (u'                        A date time range requires at minimum start '
       u'or end to'),
      (u'                        time of the boundary and END defines the '
       u'end time.'),
      (u'                        Both timestamps be set. The date time '
       u'values are'),
      (u'                        formatted as: YYYY-MM-DD hh:mm:ss.######'
       u'[+-]##:##'),
      (u'                        Where # are numeric digits ranging from 0 '
       u'to 9 and the'),
      (u'                        seconds fraction can be either 3 or 6 '
       u'digits. The time'),
      (u'                        of day, seconds fraction and timezone '
       u'offset are'),
      (u'                        optional. The default timezone is UTC. '
       u'E.g. "atime,'),
      (u'                        2013-01-01 23:12:14, 2013-02-23". This '
       u'parameter can'),
      (u'                        be repeated as needed to add additional '
       u'date'),
      (u'                        boundaries, e.g. once for atime, once for '
       u'crtime, etc.'),
      u''])

  def testAddArguments(self):
    """Tests the AddArguments function."""
    argument_parser = argparse.ArgumentParser(
        prog=u'cli_helper.py', description=u'Test argument parser.',
        add_help=False,
        formatter_class=cli_test_lib.SortedArgumentsHelpFormatter)

    date_filters.DateFiltersArgumentsHelper.AddArguments(argument_parser)

    output = self._RunArgparseFormatHelp(argument_parser)
    self.assertEqual(output, self._EXPECTED_OUTPUT)

  def testParseOptions(self):
    """Tests the ParseOptions function."""
    options = cli_test_lib.TestOptions()
    options.date_filters = [u'ctime,2012-05-25 15:59:00,2012-05-25 15:59:20']

    test_tool = tools.CLITool()

    with self.assertRaises(errors.BadConfigObject):
      date_filters.DateFiltersArgumentsHelper.ParseOptions(options, None)

    with self.assertRaises(errors.BadConfigObject):
      test_tool._filter_collection = None
      date_filters.DateFiltersArgumentsHelper.ParseOptions(options, test_tool)

    test_tool._filter_collection = (
        file_entry_filters.FileEntryFilterCollection())

    date_filters.DateFiltersArgumentsHelper.ParseOptions(options, test_tool)
    self.assertTrue(test_tool._filter_collection.HasFilters())

    with self.assertRaises(errors.BadConfigOption):
      options.date_filters = [u'ctime,2012-05-25 15:59:00']
      date_filters.DateFiltersArgumentsHelper.ParseOptions(options, test_tool)

    with self.assertRaises(errors.BadConfigOption):
      options.date_filters = [u'ctime,2012-05-25 15:59:00,2012-05-A5 15:59:20']
      date_filters.DateFiltersArgumentsHelper.ParseOptions(options, test_tool)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The preprocess plugins manager."""

import logging

from dfvfs.helpers import file_system_searcher
from dfvfs.helpers import windows_path_resolver
from dfwinreg import interface as dfwinreg_interface
from dfwinreg import regf as dfwinreg_regf
from dfwinreg import registry as dfwinreg_registry
from dfwinreg import registry_searcher

from plaso.lib import errors
from plaso.preprocessors import interface


class FileSystemWinRegistryFileReader(dfwinreg_interface.WinRegistryFileReader):
  """A file system-based Windows Registry file reader."""

  def __init__(self, file_system, mount_point, environment_variables=None):
    """Initializes a Windows Registry file reader object.

    Args:
      file_system (dfvfs.FileSytem): file system.
      mount_point (dfvfs.PathSpec): mount point path specification.
      environment_variables (Optional[list[EnvironmentVariableArtifact]]):
          environment variables.
    """
    super(FileSystemWinRegistryFileReader, self).__init__()
    self._file_system = file_system
    self._path_resolver = self._CreateWindowsPathResolver(
        file_system, mount_point, environment_variables=environment_variables)

  def _CreateWindowsPathResolver(
      self, file_system, mount_point, environment_variables):
    """Create a Windows path resolver and sets the evironment variables.

    Args:
      file_system (dfvfs.FileSytem): file system.
      mount_point (dfvfs.PathSpec): mount point path specification.
      environment_variables (list[EnvironmentVariableArtifact]): environment
          variables.

    Returns:
      dfvfs.WindowsPathResolver: Windows path resolver.
    """
    if environment_variables is None:
      environment_variables = []

    path_resolver = windows_path_resolver.WindowsPathResolver(
        file_system, mount_point)

    for environment_variable in environment_variables:
      name = environment_variable.name.lower()
      if name not in (u'systemroot', u'userprofile'):
        continue

      path_resolver.SetEnvironmentVariable(
          environment_variable.name, environment_variable.value)

    return path_resolver

  def _OpenPathSpec(self, path_specification, ascii_codepage=u'cp1252'):
    """Opens the Windows Registry file specified by the path specification.

    Args:
      path_specification (dfvfs.PathSpec): path specfication.
      ascii_codepage (Optional[str]): ASCII string codepage.

    Returns:
      WinRegistryFile: Windows Registry file or None.
    """
    if not path_specification:
      return

    file_entry = self._file_system.GetFileEntryByPathSpec(path_specification)
    if file_entry is None:
      return

    file_object = file_entry.GetFileObject()
    if file_object is None:
      return

    registry_file = dfwinreg_regf.REGFWinRegistryFile(
        ascii_codepage=ascii_codepage)

    try:
      registry_file.Open(file_object)
    except IOError as exception:
      logging.warning(
          u'Unable to open Windows Registry file with error: {0:s}'.format(
              exception))
      file_object.close()
      return

    return registry_file

  def Open(self, path, ascii_codepage=u'cp1252'):
    """Opens the Windows Registry file specified by the path.

    Args:
      path (str): path of the Windows Registry file.
      ascii_codepage (Optional[str]): ASCII string codepage.

    Returns:
      WinRegistryFile: Windows Registry file or None.
    """
    path_specification = self._path_resolver.ResolvePath(path)
    if path_specification is None:
      return

    return self._OpenPathSpec(path_specification)


class PreprocessPluginsManager(object):
  """Preprocess plugins manager."""

  _plugins = {}
  _file_system_plugins = {}
  _registry_value_plugins = {}

  @classmethod
  def CollectFromFileSystem(
      cls, artifacts_registry, knowledge_base, searcher, file_system):
    """Collects values from Windows Registry values.

    Args:
      artifacts_registry (artifacts.ArtifactDefinitionsRegistry): artifacts
          definitions registry.
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      searcher (dfvfs.FileSystemSearcher): file system searcher to preprocess
          the file system.
      file_system (dfvfs.FileSystem): file system to be preprocessed.
    """
    for preprocess_plugin in cls._file_system_plugins.values():
      artifact_definition = artifacts_registry.GetDefinitionByName(
          preprocess_plugin.ARTIFACT_DEFINITION_NAME)
      if not artifact_definition:
        logging.warning(u'Missing artifact definition: {0:s}'.format(
            preprocess_plugin.ARTIFACT_DEFINITION_NAME))
        continue

      try:
        preprocess_plugin.Collect(
            knowledge_base, artifact_definition, searcher, file_system)
      except (IOError, errors.PreProcessFail) as exception:
        logging.warning((
            u'Unable to collect value from artifact definition: {0:s} '
            u'with error: {1:s}').format(
                preprocess_plugin.ARTIFACT_DEFINITION_NAME, exception))
        continue

  @classmethod
  def CollectFromWindowsRegistry(
      cls, artifacts_registry, knowledge_base, searcher):
    """Collects values from Windows Registry values.

    Args:
      artifacts_registry (artifacts.ArtifactDefinitionsRegistry): artifacts
          definitions registry.
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      searcher (dfwinreg.WinRegistrySearcher): Windows Registry searcher to
          preprocess the Windows Registry.
    """
    for preprocess_plugin in cls._registry_value_plugins.values():
      artifact_definition = artifacts_registry.GetDefinitionByName(
          preprocess_plugin.ARTIFACT_DEFINITION_NAME)
      if not artifact_definition:
        logging.warning(u'Missing artifact definition: {0:s}'.format(
            preprocess_plugin.ARTIFACT_DEFINITION_NAME))
        continue

      try:
        preprocess_plugin.Collect(knowledge_base, artifact_definition, searcher)
      except (IOError, errors.PreProcessFail) as exception:
        logging.warning((
            u'Unable to collect value from artifact definition: {0:s} '
            u'with error: {1:s}').format(
                preprocess_plugin.ARTIFACT_DEFINITION_NAME, exception))
        continue

  @classmethod
  def DeregisterPlugin(cls, plugin_class):
    """Deregisters an preprocess plugin class.

    Args:
      preprocess_plugin (type): preprocess plugin class.

    Raises:
      KeyError: if plugin class is not set for the corresponding name.
      TypeError: if the source type of the plugin class is not supported.
    """
    name = plugin_class.ARTIFACT_DEFINITION_NAME.lower()
    if name not in cls._plugins:
      raise KeyError(
          u'Artifact plugin class not set for name: {0:s}.'.format(name))

    del cls._plugins[name]

    if name in cls._file_system_plugins:
      del cls._file_system_plugins[name]

    if name in cls._registry_value_plugins:
      del cls._registry_value_plugins[name]

  @classmethod
  def GetNames(cls):
    """Retrieves the names of the registered artifact definitions.

    Returns:
      list[str]: registered artifact definitions names.
    """
    return [
        plugin_class.ARTIFACT_DEFINITION_NAME
        for plugin_class in cls._plugins.values()]

  @classmethod
  def RegisterPlugin(cls, plugin_class):
    """Registers an preprocess plugin class.

    Args:
      plugin_class (type): preprocess plugin class.

    Raises:
      KeyError: if plugin class is already set for the corresponding name.
      TypeError: if the source type of the plugin class is not supported.
    """
    name = plugin_class.ARTIFACT_DEFINITION_NAME.lower()
    if name in cls._plugins:
      raise KeyError(
          u'Artifact plugin class already set for name: {0:s}.'.format(name))

    preprocess_plugin = plugin_class()

    cls._plugins[name] = preprocess_plugin

    if isinstance(
        preprocess_plugin, interface.FileSystemArtifactPreprocessorPlugin):
      cls._file_system_plugins[name] = preprocess_plugin

    elif isinstance(
        preprocess_plugin,
        interface.WindowsRegistryValueArtifactPreprocessorPlugin):
      cls._registry_value_plugins[name] = preprocess_plugin

  @classmethod
  def RegisterPlugins(cls, plugin_classes):
    """Registers preprocess plugin classes.

    Args:
      plugin_classes (list[type]): preprocess plugin classses.

    Raises:
      KeyError: if plugin class is already set for the corresponding name.
    """
    for plugin_class in plugin_classes:
      cls.RegisterPlugin(plugin_class)

  @classmethod
  def RunPlugins(
      cls, artifacts_registry, file_system, mount_point, knowledge_base):
    """Runs the preprocessing plugins.

    Args:
      artifacts_registry (artifacts.ArtifactDefinitionsRegistry): artifacts
          definitions registry.
      file_system (dfvfs.FileSystem): file system to be preprocessed.
      mount_point (dfvfs.PathSpec): mount point path specification that refers
          to the base location of the file system.
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
    """
    searcher = file_system_searcher.FileSystemSearcher(file_system, mount_point)

    cls.CollectFromFileSystem(
        artifacts_registry, knowledge_base, searcher, file_system)

    # Run the Registry plugins separately so we do not have to open
    # Registry files for every preprocess plugin.

    environment_variables = None
    if knowledge_base:
      environment_variables = knowledge_base.GetEnvironmentVariables()

    registry_file_reader = FileSystemWinRegistryFileReader(
        file_system, mount_point, environment_variables=environment_variables)
    win_registry = dfwinreg_registry.WinRegistry(
        registry_file_reader=registry_file_reader)

    searcher = registry_searcher.WinRegistrySearcher(win_registry)

    cls.CollectFromWindowsRegistry(
        artifacts_registry, knowledge_base, searcher)

    if not knowledge_base.HasUserAccounts():
      logging.warning(u'Unable to find any user accounts on the system.')



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the PE file parser."""

import unittest

from plaso.formatters import pe  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers import pe

from tests import test_lib as shared_test_lib
from tests.parsers import test_lib


class PECOFFTest(test_lib.ParserTestCase):
  """Tests for the PE file parser."""

  @shared_test_lib.skipUnlessHasTestFile([u'test_pe.exe'])
  def testParseFileObjectOnExecutable(self):
    """Tests the ParseFileObject on a PE executable (EXE) file."""
    parser = pe.PEParser()
    storage_writer = self._ParseFile([u'test_pe.exe'], parser)

    self.assertEqual(storage_writer.number_of_events, 3)

    events = list(storage_writer.GetEvents())

    event = events[0]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2015-04-21 14:53:56')
    self.assertEqual(event.pe_type, u'Executable (EXE)')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.data_type, u'pe:compilation:compilation_time')

    event = events[1]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2015-04-21 14:53:55')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.data_type, u'pe:import:import_time')

    event = events[2]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2015-04-21 14:53:54')
    self.assertEqual(event.timestamp, expected_timestamp)
    self.assertEqual(event.data_type, u'pe:delay_import:import_time')

  @shared_test_lib.skipUnlessHasTestFile([u'test_driver.sys'])
  def testParseFileObjectOnDriver(self):
    """Tests the ParseFileObject on a PE driver (SYS) file."""
    parser = pe.PEParser()
    storage_writer = self._ParseFile([u'test_driver.sys'], parser)

    self.assertEqual(storage_writer.number_of_events, 1)

    events = list(storage_writer.GetEvents())

    event = events[0]
    expected_timestamp = timelib.Timestamp.CopyFromString(
        u'2015-04-21 14:53:54')
    self.assertEqual(event.pe_type, u'Driver (SYS)')
    self.assertEqual(event.timestamp, expected_timestamp)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Bash history event formatter."""
from plaso.formatters import interface
from plaso.formatters import manager


class BashHistoryEventFormatter(interface.EventFormatter):
  """Formatter for Bash history events."""

  DATA_TYPE = u'bash:history:command'

  FORMAT_STRING = u'Command executed: {command}'
  FORMAT_STRING_SHORT = u'{command}'

  SOURCE_SHORT = u'LOG'
  SOURCE_LONG = u'Bash History'


manager.FormattersManager.RegisterFormatter(BashHistoryEventFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the XLSX output module."""

import os
import unittest
import zipfile

from xml.etree import ElementTree

from plaso.containers import events
from plaso.formatters import interface as formatters_interface
from plaso.formatters import manager as formatters_manager
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.output import xlsx

from tests import test_lib as shared_test_lib
from tests.output import test_lib


class TestEvent(events.EventObject):
  """Event object used for testing."""
  DATA_TYPE = u'test:xlsx'

  def __init__(self):
    """Initializes an event object used for testing."""
    super(TestEvent, self).__init__()
    self.timestamp = timelib.Timestamp.CopyFromString(u'2012-06-27 18:17:01')
    self.timestamp_desc = definitions.TIME_DESCRIPTION_CHANGE
    self.hostname = u'ubuntu'
    self.filename = u'log/syslog.1'
    self.text = (
        u'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session\n '
        u'closed for user root) Invalid character -> \ud801')


class TestEventFormatter(formatters_interface.EventFormatter):
  """Event object formatter used for testing."""

  DATA_TYPE = u'test:xlsx'
  FORMAT_STRING = u'{text}'

  SOURCE_SHORT = u'LOG'
  SOURCE_LONG = u'Syslog'


class XLSXOutputModuleTest(test_lib.OutputModuleTestCase):
  """Test the XLSX output module."""

  _SHARED_STRINGS = u'xl/sharedStrings.xml'
  _SHEET1 = u'xl/worksheets/sheet1.xml'

  _COLUMN_TAG = u'}c'
  _ROW_TAG = u'}row'
  _SHARED_STRING_TAG = u'}t'
  _SHARED_STRING_TYPE = u's'
  _TYPE_ATTRIBUTE = u't'
  _VALUE_STRING_TAG = u'}v'

  def _GetSheetRows(self, filename):
    """Parses the contents of the first sheet of an XLSX document.

    Args:
      filename: The file path of the XLSX document to parse.

    Returns:
      A list of dictionaries representing the rows and columns of the first
      sheet.
    """
    zip_file = zipfile.ZipFile(filename)

    # Fail if we can't find the expected first sheet.
    if self._SHEET1 not in zip_file.namelist():
      raise ValueError(
          u'Unable to locate expected sheet: {0:s}'.format(self._SHEET1))

    # Generate a reference table of shared strings if available.
    strings = []
    if self._SHARED_STRINGS in zip_file.namelist():
      zip_file_object = zip_file.open(self._SHARED_STRINGS)
      for _, element in ElementTree.iterparse(zip_file_object):
        if element.tag.endswith(self._SHARED_STRING_TAG):
          strings.append(element.text)

    row = []
    rows = []
    value = u''
    zip_file_object = zip_file.open(self._SHEET1)
    for _, element in ElementTree.iterparse(zip_file_object):
      if (element.tag.endswith(self._VALUE_STRING_TAG) or
          element.tag.endswith(self._SHARED_STRING_TAG)):
        value = element.text

      if element.tag.endswith(self._COLUMN_TAG):
        # Grab value from shared string reference table if type shared string.
        if (strings and element.attrib.get(self._TYPE_ATTRIBUTE) ==
            self._SHARED_STRING_TYPE):
          try:
            value = strings[int(value)]
          except (IndexError, ValueError):
            raise ValueError(
                u'Unable to successfully dereference shared string.')

        row.append(value)

      # If we see the end tag of the row, record row in rows and reset.
      if element.tag.endswith(self._ROW_TAG):
        rows.append(row)
        row = []

    return rows

  def testWriteEventBody(self):
    """Tests the WriteHeader function."""
    formatters_manager.FormattersManager.RegisterFormatter(TestEventFormatter)

    expected_header = [
        u'datetime', u'timestamp_desc', u'source', u'source_long',
        u'message', u'parser', u'display_name', u'tag']
    expected_event_body = [
        u'41087.76181712963', u'Metadata Modification Time', u'LOG', u'Syslog',
        u'Reporter <CRON> PID: 8442 (pam_unix(cron:session): session '
        u'closed for user root) Invalid character -> \ufffd',
        u'-', u'-', u'-']

    with shared_test_lib.TempDirectory() as temp_directory:
      output_mediator = self._CreateOutputMediator()
      output_module = xlsx.XLSXOutputModule(output_mediator)

      xslx_file = os.path.join(temp_directory, u'xlsx.out')
      output_module.SetFilename(xslx_file)

      output_module.Open()
      output_module.WriteHeader()
      output_module.WriteEvent(TestEvent())
      output_module.WriteFooter()
      output_module.Close()

      try:
        rows = self._GetSheetRows(xslx_file)
      except ValueError as exception:
        self.fail(exception)

      self.assertEqual(expected_header, rows[0])
      self.assertEqual(len(expected_event_body), len(rows[1]))
      self.assertEqual(expected_event_body, rows[1])

  def testWriteHeader(self):
    """Tests the WriteHeader function."""
    expected_header = [
        u'datetime', u'timestamp_desc', u'source', u'source_long',
        u'message', u'parser', u'display_name', u'tag']

    with shared_test_lib.TempDirectory() as temp_directory:
      output_mediator = self._CreateOutputMediator()
      output_module = xlsx.XLSXOutputModule(output_mediator)

      xlsx_file = os.path.join(temp_directory, u'xlsx.out')
      output_module.SetFilename(xlsx_file)

      output_module.Open()
      output_module.WriteHeader()
      output_module.WriteFooter()
      output_module.Close()

      try:
        rows = self._GetSheetRows(xlsx_file)
      except ValueError as exception:
        self.fail(exception)

      self.assertEqual(expected_header, rows[0])


if __name__ == u'__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The dynamic event object filter."""

from plaso.filters import event_filter
from plaso.filters import manager
from plaso.lib import errors
from plaso.lib import lexer


# TODO: move this to lib.lexer ?
class SelectiveLexer(lexer.Lexer):
  """Selective filter lexer implementation.

  The selective (or dynamic) filter allow to construct filter expressions
  like:
    SELECT field_a, field_b WHERE attribute contains 'text'
  """

  tokens = [
      lexer.Token('INITIAL', r'SELECT', '', 'FIELDS'),
      lexer.Token('FIELDS', r'(.+) WHERE ', 'SetFields', 'FILTER'),
      lexer.Token('FIELDS', r'(.+) LIMIT', 'SetFields', 'LIMIT_END'),
      lexer.Token('FIELDS', r'(.+) SEPARATED BY', 'SetFields', 'SEPARATE'),
      lexer.Token('FIELDS', r'(.+)$', 'SetFields', 'END'),
      lexer.Token('FILTER', r'(.+) SEPARATED BY', 'SetFilter', 'SEPARATE'),
      lexer.Token('FILTER', r'(.+) LIMIT', 'SetFilter', 'LIMIT_END'),
      lexer.Token('FILTER', r'(.+)$', 'SetFilter', 'END'),
      lexer.Token('SEPARATE', r' ', '', ''),  # Ignore white space here.
      lexer.Token('SEPARATE', r'LIMIT', '', 'LIMIT_END'),
      lexer.Token(
          'SEPARATE', r'[\'"]([^ \'"]+)[\'"] LIMIT', 'SetSeparator',
          'LIMIT_END'),
      lexer.Token(
          'SEPARATE', r'[\'"]([^ \'"]+)[\'"]$', 'SetSeparator', 'END'),
      lexer.Token(
          'SEPARATE', r'(.+)$', 'SetSeparator', 'END'),
      lexer.Token(
          'LIMIT_END', r'SEPARATED BY [\'"]([^\'"]+)[\'"]', 'SetSeparator', ''),
      lexer.Token('LIMIT_END', r'(.+) SEPARATED BY', 'SetLimit', 'SEPARATE'),
      lexer.Token('LIMIT_END', r'(.+)$', 'SetLimit', 'END')]

  def __init__(self, data=''):
    """Initializes a selective lexer object.

    Args:
      data: optional initial data to be processed by the lexer.
    """
    super(SelectiveLexer, self).__init__(data=data)
    self.fields = []
    self.limit = 0
    self.lex_filter = None
    self.separator = u','

  def SetFields(self, match, **unused_kwargs):
    """Sets the output fields.

    The output fields is the part of the filter expression directly following
    the SELECT statement.

    Args:
      match: the match object (instance of re.MatchObject) that contains the
             output field names.
    """
    text = match.group(1).lower()
    field_text, _, _ = text.partition(' from ')

    use_field_text = field_text.replace(' ', '')
    if ',' in use_field_text:
      self.fields = use_field_text.split(',')
    else:
      self.fields = [use_field_text]

  def SetFilter(self, match, **unused_kwargs):
    """Set the filter query.

    The filter query is the part of the filter expression directly following
    the WHERE statement.

    Args:
      match: the match object (instance of re.MatchObject) that contains the
             filter query.
    """
    filter_match = match.group(1)
    if 'LIMIT' in filter_match:
      # This only occurs in the case where we have "LIMIT X SEPARATED BY".
      self.lex_filter, _, push_back = filter_match.rpartition('LIMIT')
      self.PushBack('LIMIT {0:s} SEPARATED BY '.format(push_back))
    else:
      self.lex_filter = filter_match

  def SetLimit(self, match, **unused_kwargs):
    """Sets the row limit.

    Args:
      match: the match object (instance of re.MatchObject) that contains the
             row limit.
    """
    try:
      limit = int(match.group(1))
    except ValueError:
      self.Error('Invalid limit value, should be int [{}] = {}'.format(
          type(match.group(1)), match.group(1)))
      limit = 0

    self.limit = limit

  def SetSeparator(self, match, **unused_kwargs):
    """Sets the output field separator.

    Args:
      match: the match object (instance of re.MatchObject) that contains the
             output field separate. Note that only the first character is used.
    """
    separator = match.group(1)
    if separator:
      self.separator = separator[0]


class DynamicFilter(event_filter.EventObjectFilter):
  """Event object filter that supports selective output fields.

  This filter is essentially the same as the event object filter except it wraps
  it in a selection of which fields should be included by an output module that
  supports selective fields, e.g.

    SELECT field_a, field_b WHERE attribute contains 'text'

  Will use the event object filter "attribute contains 'text'" and at the same
  time indicate to the appropriate output module that the user wants only the
  fields field_a and field_b to be used in the output.
  """

  _STATE_END = u'END'

  def __init__(self):
    """Initializes a filter object."""
    super(DynamicFilter, self).__init__()
    self._fields = []
    self._limit = 0
    self._separator = u','

  @property
  def fields(self):
    """The output fields."""
    return self._fields

  @property
  def limit(self):
    """The row limit."""
    return self._limit

  @property
  def separator(self):
    """The output field separator value."""
    return self._separator

  def CompileFilter(self, filter_expression):
    """Compiles the filter expression.

    The filter expression contains an object filter expression extended
    with selective field selection.

    Args:
      filter_expression: string that contains the filter expression.

    Raises:
      WrongPlugin: if the filter could not be compiled.
    """
    lexer_object = SelectiveLexer(filter_expression)

    lexer_object.NextToken()
    if lexer_object.error:
      raise errors.WrongPlugin(u'Malformed filter string.')

    lexer_object.NextToken()
    if lexer_object.error:
      raise errors.WrongPlugin(u'No fields defined.')

    while lexer_object.state != self._STATE_END:
      lexer_object.NextToken()
      if lexer_object.error:
        raise errors.WrongPlugin(u'No filter defined for DynamicFilter.')

    if lexer_object.state != self._STATE_END:
      raise errors.WrongPlugin(
          u'Malformed DynamicFilter, end state not reached.')

    self._fields = lexer_object.fields
    self._limit = lexer_object.limit
    self._separator = u'{0:s}'.format(lexer_object.separator)

    if lexer_object.lex_filter:
      super(DynamicFilter, self).CompileFilter(lexer_object.lex_filter)
    else:
      self._matcher = None
    self._filter_expression = filter_expression


manager.FiltersManager.RegisterFilter(DynamicFilter)



# -*- coding: utf-8 -*-
"""This file contains preprocessors for Windows."""

import logging

from plaso.containers import artifacts
from plaso.lib import errors
from plaso.lib import py2to3
from plaso.preprocessors import interface
from plaso.preprocessors import manager
from plaso.winnt import time_zones


class WindowsEnvironmentVariableArtifactPreprocessorPlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """Windows environment variable artifact preprocessor plugin interface."""

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Windows Registry value type: {0:s} for '
          u'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    result = False
    evironment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name=self._NAME, value=value_data)

    try:
      knowledge_base.AddEnvironmentVariable(evironment_variable)
      result = True
    except KeyError:
      # TODO: add and store preprocessing errors.
      pass

    return result


class WindowsPathEnvironmentVariableArtifactPreprocessorPlugin(
    interface.FileSystemArtifactPreprocessorPlugin):
  """Windows path environment variable plugin interface."""

  _NAME = None

  def _ParsePathSpecification(
      self, knowledge_base, searcher, file_system, path_specification,
      path_separator):
    """Parses artifact file system data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      searcher (dfvfs.FileSystemSearcher): file system searcher to preprocess
          the file system.
      file_system (dfvfs.FileSystem): file system to be preprocessed.
      path_specification (dfvfs.PathSpec): path specification that contains
          the artifact value data.
      path_separator (str): path segment separator.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    result = False

    relative_path = searcher.GetRelativePath(path_specification)
    if not relative_path:
      raise errors.PreProcessFail(
          u'Unable to read: {0:s} with error: missing relative path'.format(
              self.ARTIFACT_DEFINITION_NAME))

    if path_separator != file_system.PATH_SEPARATOR:
      relative_path_segments = file_system.SplitPath(relative_path)
      relative_path = u'{0:s}{1:s}'.format(
          path_separator, path_separator.join(relative_path_segments))

    evironment_variable = artifacts.EnvironmentVariableArtifact(
        case_sensitive=False, name=self._NAME, value=relative_path)

    try:
      knowledge_base.AddEnvironmentVariable(evironment_variable)
      result = True
    except KeyError:
      # TODO: add and store preprocessing errors.
      pass

    return result


class WindowsCodepagePlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """The Windows codepage plugin."""

  ARTIFACT_DEFINITION_NAME = u'WindowsCodePage'

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Windows Registry value type: {0:s} for '
          u'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    result = False
    # Map the Windows code page name to a Python equivalent name.
    codepage = u'cp{0:s}'.format(value_data)

    try:
      knowledge_base.SetCodepage(codepage)
      result = True
    except ValueError:
      # TODO: add and store preprocessing errors.
      pass

    return result


class WindowsHostnamePlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """The Windows hostname plugin."""

  ARTIFACT_DEFINITION_NAME = u'WindowsComputerName'

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Windows Registry value type: {0:s} for '
          u'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    hostname_artifact = artifacts.HostnameArtifact(name=value_data)
    knowledge_base.SetHostname(hostname_artifact)
    return True


class WindowsProgramFilesEnvironmentVariablePlugin(
    WindowsEnvironmentVariableArtifactPreprocessorPlugin):
  """The Windows %ProgramFiles% environment variable plugin."""

  ARTIFACT_DEFINITION_NAME = u'WindowsEnvironmentVariableProgramFiles'

  _NAME = u'programfiles'


class WindowsProgramFilesX86EnvironmentVariablePlugin(
    WindowsEnvironmentVariableArtifactPreprocessorPlugin):
  """The Windows %ProgramFilesX86% environment variable plugin."""

  ARTIFACT_DEFINITION_NAME = u'WindowsEnvironmentVariableProgramFilesX86'

  _NAME = u'programfilesx86'


class WindowsSystemProductPlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """The Windows system product information plugin."""

  ARTIFACT_DEFINITION_NAME = u'WindowsProductName'

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Windows Registry value type: {0:s} for '
          u'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    knowledge_base.SetValue(u'operating_system_product', value_data)
    return True


class WindowsSystemRootEnvironmentVariablePlugin(
    WindowsPathEnvironmentVariableArtifactPreprocessorPlugin):
  """The Windows %SystemRoot% environment variable plugin."""

  ARTIFACT_DEFINITION_NAME = u'WindowsEnvironmentVariableSystemRoot'

  _NAME = u'systemroot'


class WindowsSystemVersionPlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """The Windows system verion information plugin."""

  ARTIFACT_DEFINITION_NAME = u'WindowsCurrentVersion'

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Windows Registry value type: {0:s} for '
          u'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    knowledge_base.SetValue(u'operating_system_version', value_data)
    return True


class WindowsTimeZonePlugin(
    interface.WindowsRegistryValueArtifactPreprocessorPlugin):
  """The Windows time zone plugin."""

  ARTIFACT_DEFINITION_NAME = u'WindowsTimezone'

  def _ParseValueData(self, knowledge_base, value_data):
    """Parses Windows Registry value data for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      value_data (object): Windows Registry value data.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    if not isinstance(value_data, py2to3.UNICODE_TYPE):
      raise errors.PreProcessFail(
          u'Unsupported Windows Registry value type: {0:s} for '
          u'artifact: {1:s}.'.format(
              type(value_data), self.ARTIFACT_DEFINITION_NAME))

    result = False
    # Map the Windows time zone name to a Python equivalent name.
    lookup_key = value_data.replace(u' ', u'')

    time_zone = time_zones.TIME_ZONES.get(lookup_key, value_data)
    if not time_zone:
      return

    try:
      # Catch and warn about unsupported preprocessor plugin.
      knowledge_base.SetTimeZone(time_zone)
      result = True
    except ValueError:
      # TODO: add and store preprocessing errors.
      time_zone = value_data
      logging.warning(u'Unable to map: "{0:s}" to time zone'.format(value_data))

    return result


class WindowsUserAccountsPlugin(
    interface.WindowsRegistryKeyArtifactPreprocessorPlugin):
  """The Windows user account plugin."""

  ARTIFACT_DEFINITION_NAME = u'WindowsRegistryProfiles'

  def _GetUsernameFromProfilePath(self, path):
    """Retrieves the username from a Windows profile path.

    Trailing path path segment are igored.

    Args:
      path (str): a Windows path with '\\' as path segment separator.

    Returns:
      str: basename which is the last path segment.
    """
    # Strip trailing key separators.
    while path and path[-1] == u'\\':
      path = path[:-1]

    if path:
      _, _, path = path.rpartition(u'\\')
    return path

  def _ParseKey(self, knowledge_base, registry_key, value_name):
    """Parses a Windows Registry key for a preprocessing attribute.

    Args:
      knowledge_base (KnowledgeBase): to fill with preprocessing information.
      registry_key (dfwinreg.WinRegistryKey): Windows Registry key.
      value_name (str): name of the Windows Registry value.

    Returns:
      bool: True if all the preprocessing attributes were found and
          the preprocessor plugin is done.

    Raises:
      errors.PreProcessFail: if the preprocessing fails.
    """
    user_account = artifacts.UserAccountArtifact(
        identifier=registry_key.name)

    registry_value = registry_key.GetValueByName(u'ProfileImagePath')
    if registry_value:
      profile_path = registry_value.GetDataAsObject()
      username = self._GetUsernameFromProfilePath(profile_path)

      user_account.user_directory = profile_path or None
      user_account.username = username or None

    try:
      knowledge_base.AddUserAccount(user_account)
    except KeyError:
      # TODO: add and store preprocessing errors.
      pass

    return False


class WindowsWinDirEnvironmentVariablePlugin(
    WindowsPathEnvironmentVariableArtifactPreprocessorPlugin):
  """The Windows %WinDir% environment variable plugin."""

  ARTIFACT_DEFINITION_NAME = u'WindowsEnvironmentVariableWinDir'

  _NAME = u'windir'


manager.PreprocessPluginsManager.RegisterPlugins([
    WindowsCodepagePlugin, WindowsHostnamePlugin,
    WindowsProgramFilesEnvironmentVariablePlugin,
    WindowsProgramFilesX86EnvironmentVariablePlugin,
    WindowsSystemProductPlugin, WindowsSystemRootEnvironmentVariablePlugin,
    WindowsSystemVersionPlugin, WindowsTimeZonePlugin,
    WindowsWinDirEnvironmentVariablePlugin, WindowsUserAccountsPlugin])



# -*- coding: utf-8 -*-
"""Output module for the log2timeline (L2T) CSV format.

For documentation on the L2T CSV format see:
http://forensicswiki.org/wiki/L2T_CSV
"""

import logging

from plaso.lib import definitions
from plaso.lib import errors
from plaso.lib import py2to3
from plaso.lib import timelib
from plaso.output import interface
from plaso.output import manager


class L2TCSVOutputModule(interface.LinearOutputModule):
  """CSV format used by log2timeline, with 17 fixed fields."""

  NAME = u'l2tcsv'
  DESCRIPTION = u'CSV format used by legacy log2timeline, with 17 fixed fields.'

  _FIELD_DELIMITER = u','
  _HEADER = (
      u'date,time,timezone,MACB,source,sourcetype,type,user,host,short,desc,'
      u'version,filename,inode,notes,format,extra\n')

  def _FormatField(self, field):
    """Formats a field.

    Args:
      field (str): field value.

     Returns:
       str: formatted field value.
    """
    if self._FIELD_DELIMITER and isinstance(field, py2to3.STRING_TYPES):
      return field.replace(self._FIELD_DELIMITER, u' ')
    return field

  def _FormatHostname(self, event):
    """Formats the hostname.

    Args:
      event (EventObject): event.

     Returns:
       str: formatted hostname field.
    """
    hostname = self._output_mediator.GetHostname(event)
    return self._FormatField(hostname)

  def _FormatUsername(self, event):
    """Formats the username.

    Args:
      event (EventObject): event.

     Returns:
       str: formatted username field.
    """
    username = self._output_mediator.GetUsername(event)
    return self._FormatField(username)

  def _WriteOutputValues(self, output_values):
    """Writes values to the output.

    Args:
      output_values (list[str]): output values.
    """
    for index, value in enumerate(output_values):
      if not isinstance(value, py2to3.STRING_TYPES):
        value = u''
      output_values[index] = value.replace(u',', u' ')

    output_line = u','.join(output_values)
    output_line = u'{0:s}\n'.format(output_line)
    self._WriteLine(output_line)

  def _GetOutputValues(self, event):
    """Retrieves output values.

    Args:
      event (EventObject): event.

    Returns:
      list[str]: output values.

    Raises:
      NoFormatterFound: If no event formatter can be found to match the data
          type in the event.
    """
    if not hasattr(event, u'timestamp'):
      logging.error(u'Unable to output event without timestamp.')
      return

    # TODO: add function to pass event_values to GetFormattedMessages.
    message, message_short = self._output_mediator.GetFormattedMessages(event)
    if message is None or message_short is None:
      data_type = getattr(event, u'data_type', u'UNKNOWN')
      raise errors.NoFormatterFound(
          u'Unable to find event formatter for: {0:s}.'.format(data_type))

    # TODO: add function to pass event_values to GetFormattedSources.
    source_short, source = self._output_mediator.GetFormattedSources(event)
    if source is None or source_short is None:
      data_type = getattr(event, u'data_type', u'UNKNOWN')
      raise errors.NoFormatterFound(
          u'Unable to find event formatter for: {0:s}.'.format(data_type))

    date_use = timelib.Timestamp.CopyToDatetime(
        event.timestamp, self._output_mediator.timezone)

    format_variables = self._output_mediator.GetFormatStringAttributeNames(
        event)
    if format_variables is None:
      data_type = getattr(event, u'data_type', u'UNKNOWN')
      raise errors.NoFormatterFound(
          u'Unable to find event formatter for: {0:s}.'.format(data_type))

    extra_attributes = []
    for attribute_name, attribute_value in sorted(event.GetAttributes()):
      if (attribute_name in definitions.RESERVED_VARIABLE_NAMES or
          attribute_name in format_variables):
        continue

      # With ! in {1!s} we force a string conversion since some of
      # the extra attributes values can be integer, float point or
      # boolean values.
      extra_attributes.append(
          u'{0:s}: {1!s}'.format(attribute_name, attribute_value))

    extra_attributes = u'; '.join(extra_attributes)
    extra_attributes = extra_attributes.replace(u'\n', u'-').replace(u'\r', u'')

    inode = getattr(event, u'inode', None)
    if inode is None:
      if hasattr(event, u'pathspec') and hasattr(
          event.pathspec, u'image_inode'):
        inode = event.pathspec.image_inode
    if inode is None:
      inode = u'-'

    hostname = self._FormatHostname(event)
    username = self._FormatUsername(event)

    notes = []
    note_string = getattr(event, u'notes', None)
    if note_string:
      notes.append(note_string)

    tag = getattr(event, u'tag', None)
    if tag:
      notes.extend(tag.labels)

    if not notes:
      notes.append(u'-')

    date_string = u'{0:02d}/{1:02d}/{2:04d}'.format(
        date_use.month, date_use.day, date_use.year)
    time_string = u'{0:02d}:{1:02d}:{2:02d}'.format(
        date_use.hour, date_use.minute, date_use.second)

    output_values = [
        date_string,
        time_string,
        u'{0!s}'.format(self._output_mediator.timezone),
        u'....',
        source_short,
        source,
        u'-',
        username,
        hostname,
        message_short,
        message,
        u'2',
        getattr(event, u'display_name', u'-'),
        u'{0!s}'.format(inode),
        u' '.join(notes),
        getattr(event, u'parser', u'-'),
        extra_attributes]

    return output_values

  def WriteEventBody(self, event):
    """Writes the body of an event object to the output.

    Args:
      event (EventObject): event.

    Raises:
      NoFormatterFound: If no event formatter can be found to match the data
          type in the event object.
    """
    output_values = self._GetOutputValues(event)

    output_values[3] = self._output_mediator.GetMACBRepresentation(event)
    output_values[6] = getattr(event, u'timestamp_desc', u'-')

    self._WriteOutputValues(output_values)

  def WriteEventMACBGroup(self, event_macb_group):
    """Writes an event MACB group to the output.

    Args:
      event_macb_group (list[EventObject]): event MACB group.
    """
    output_values = self._GetOutputValues(event_macb_group[0])

    timestamp_descriptions = [
        event.timestamp_desc for event in event_macb_group]
    output_values[3] = (
        self._output_mediator.GetMACBRepresentationFromDescriptions(
            timestamp_descriptions))
    # TODO: fix timestamp description in source.
    output_values[6] = u'; '.join(timestamp_descriptions)

    self._WriteOutputValues(output_values)

  def WriteHeader(self):
    """Writes the header to the output."""
    self._WriteLine(self._HEADER)


manager.OutputManager.RegisterOutput(L2TCSVOutputModule)



# -*- coding: utf-8 -*-
"""The Windows Registry key or value event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


class WinRegistryGenericFormatter(interface.EventFormatter):
  """Formatter for a Windows Registry key or value event."""

  DATA_TYPE = u'windows:registry:key_value'

  FORMAT_STRING = u'[{key_path}] {text}'
  FORMAT_STRING_ALTERNATIVE = u'{text}'

  SOURCE_LONG = u'Registry Key'
  SOURCE_SHORT = u'REG'

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    regvalue = event_values.get(u'regvalue', {})
    string_parts = []
    for key, value in sorted(regvalue.items()):
      string_parts.append(u'{0:s}: {1!s}'.format(key, value))
    event_values[u'text'] = u' '.join(string_parts)

    urls = event_values.get(u'urls', [])
    if urls:
      event_values[u'urls'] = u' - '.join(urls)

    if u'key_path' in event_values:
      format_string = self.FORMAT_STRING
    else:
      format_string = self.FORMAT_STRING_ALTERNATIVE

    return self._FormatMessages(
        format_string, self.FORMAT_STRING_SHORT, event_values)

  def GetSources(self, event):
    """Determines the the short and long source for an event object.

    Args:
      event (EventObject): event.

    Returns:
      tuple(str, str): short and long source string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    source_long = getattr(event, u'source_long', u'UNKNOWN')
    source_append = getattr(event, u'source_append', None)
    if source_append:
      source_long = u'{0:s} {1:s}'.format(source_long, source_append)

    return self.SOURCE_SHORT, source_long


manager.FormattersManager.RegisterFormatter(WinRegistryGenericFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the circular buffer for storing event objects."""

import unittest

from plaso.lib import bufferlib


class CircularBufferTest(unittest.TestCase):
  """Tests for the circular buffer for storing event objects."""

  def testBuffer(self):
    """Tests the circular buffer."""
    items = range(1, 11)

    circular_buffer = bufferlib.CircularBuffer(10)

    self.assertEqual(len(circular_buffer), 10)
    self.assertEqual(circular_buffer.size, 10)

    current_item = circular_buffer.GetCurrent()
    self.assertIsNone(current_item)

    for item in items:
      circular_buffer.Append(item)
      current_item = circular_buffer.GetCurrent()
      self.assertEqual(current_item, item)
      self.assertEqual(circular_buffer.size, 10)

    content = list(circular_buffer)
    self.assertEqual(items, content)

    circular_buffer.Append(11)

    expected_items = [2, 3, 4, 5, 6, 7, 8, 9, 10, 11]
    items = list(circular_buffer.Flush())
    self.assertEqual(items, expected_items)

    self.assertIsNone(circular_buffer.GetCurrent())

    items = range(1, 51)
    for item in items:
      circular_buffer.Append(item)
      self.assertEqual(circular_buffer.GetCurrent(), item)
      self.assertEqual(circular_buffer.size, 10)

    expected_items = range(41, 51)
    items = list(circular_buffer)
    self.assertEqual(items, expected_items)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Windows firewall log file event formatter."""

import unittest

from plaso.formatters import winfirewall

from tests.formatters import test_lib


class WinFirewallFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Windows firewall log entry event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = winfirewall.WinFirewallFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = winfirewall.WinFirewallFormatter()

    expected_attribute_names = [
        u'action',
        u'protocol',
        u'path',
        u'source_ip',
        u'source_port',
        u'dest_ip',
        u'dest_port',
        u'size',
        u'flags',
        u'tcp_seq',
        u'tcp_ack',
        u'tcp_win',
        u'icmp_type',
        u'icmp_code',
        u'info']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""This file contains an import statement for each hasher."""
from plaso.analyzers.hashers import md5
from plaso.analyzers.hashers import sha1
from plaso.analyzers.hashers import sha256



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the scan tree-based path filter."""

import unittest

from plaso.filters import path_filter

from tests.filters import test_lib


class PathFilterScanTreeTest(test_lib.FilterTestCase):
  """Tests for the path filter scan tree."""

  # pylint: disable=protected-access

  def testInitialize(self):
    """Tests the initialize function."""
    scan_tree = path_filter.PathFilterScanTree([])
    self.assertIsNone(scan_tree._root_node)

    paths = [
        u'HKEY_CURRENT_USER\\Software\\WinRAR\\ArcHistory',
        u'HKEY_CURRENT_USER\\Software\\WinRAR\\DialogEditHistory\\ArcName',
        u'HKEY_CURRENT_USER\\Software\\WinRAR\\DialogEditHistory\\ExtrPath',
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{FA99DFC7-6AC2-453A-A5E2-5E2AFF4507BD}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{F4E57C4B-2036-45F0-A9AB-443BCFE33D9F}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{F2A1CB5A-E3CC-4A2E-AF9D-505A7009D442}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{CEBFF5CD-ACE2-4F4F-9178-9926F41749EA}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{CAA59E3C-4792-41A5-9909-6A6A8D32490E}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{B267E3AD-A825-4A09-82B9-EEC22AA3B847}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{A3D53349-6E61-4557-8FC7-0028EDCEEBF6}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{9E04CAB2-CC14-11DF-BB8C-A2F1DED72085}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{75048700-EF1F-11D0-9888-006097DEACF9}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{5E6AB780-7743-11CF-A12B-00AA004AE837}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{0D6D4F41-2994-4BA0-8FEF-620E43CD2812}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\UserAssist\\{BCB48336-4DDD-48FF-BB0B-D3190DACB3E2}'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Internet Explorer\\'
         u'TypedURLs'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Explorer\\TypedPaths'),
        (u'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\'
         u'Session Manager\\AppCompatibility'),
        (u'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\'
         u'Session Manager\\AppCompatCache'),
        (u'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows NT\\'
         u'CurrentVersion'),
        u'HKEY_LOCAL_MACHINE\\SAM\\Domains\\Account\\Users',
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Internet Settings\\Lockdown_Zones'),
        (u'HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Internet Settings\\Zones'),
        (u'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Internet Settings\\Lockdown_Zones'),
        (u'HKEY_LOCAL_MACHINE\\Software\\Microsoft\\Windows\\CurrentVersion\\'
         u'Internet Settings\\Zones'),
    ]

    scan_tree = path_filter.PathFilterScanTree(
        paths, path_segment_separator=u'\\')
    self.assertIsNotNone(scan_tree._root_node)

    path = u'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\Windows'
    self.assertFalse(scan_tree.CheckPath(path))

    path = (
        u'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\'
        u'Session Manager\\AppCompatCache')
    self.assertTrue(scan_tree.CheckPath(path))

    path = (
        u'HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Control\\'
        u'Session Manager\\AppCompatCache')
    self.assertFalse(scan_tree.CheckPath(path))

    path = (
        u'HKEY_LOCAL_MACHINE/System/CurrentControlSet/Control/'
        u'Session Manager/AppCompatCache')
    self.assertTrue(scan_tree.CheckPath(path, path_segment_separator=u'/'))

    scan_tree = path_filter.PathFilterScanTree(
        paths, case_sensitive=False, path_segment_separator=u'\\')
    self.assertIsNotNone(scan_tree._root_node)

    path = u'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\Windows'
    self.assertFalse(scan_tree.CheckPath(path))

    path = (
        u'HKEY_LOCAL_MACHINE\\System\\CurrentControlSet\\Control\\'
        u'Session Manager\\AppCompatCache')
    self.assertTrue(scan_tree.CheckPath(path))

    path = (
        u'HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Control\\'
        u'Session Manager\\AppCompatCache')
    self.assertTrue(scan_tree.CheckPath(path))

    path = (
        u'HKEY_LOCAL_MACHINE/System/CurrentControlSet/Control/'
        u'Session Manager/AppCompatCache')
    self.assertTrue(scan_tree.CheckPath(path, path_segment_separator=u'/'))


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Symantec AV log file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager
from plaso.lib import errors


__author__ = 'David Nides (david.nides@gmail.com)'


class SymantecAVFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Symantec AV log file event."""

  DATA_TYPE = u'av:symantec:scanlog'

  EVENT_NAMES = {
      u'1': u'GL_EVENT_IS_ALERT',
      u'2': u'GL_EVENT_SCAN_STOP',
      u'3': u'GL_EVENT_SCAN_START',
      u'4': u'GL_EVENT_PATTERN_UPDATE',
      u'5': u'GL_EVENT_INFECTION',
      u'6': u'GL_EVENT_FILE_NOT_OPEN',
      u'7': u'GL_EVENT_LOAD_PATTERN',
      u'8': u'GL_STD_MESSAGE_INFO',
      u'9': u'GL_STD_MESSAGE_ERROR',
      u'10': u'GL_EVENT_CHECKSUM',
      u'11': u'GL_EVENT_TRAP',
      u'12': u'GL_EVENT_CONFIG_CHANGE',
      u'13': u'GL_EVENT_SHUTDOWN',
      u'14': u'GL_EVENT_STARTUP',
      u'16': u'GL_EVENT_PATTERN_DOWNLOAD',
      u'17': u'GL_EVENT_TOO_MANY_VIRUSES',
      u'18': u'GL_EVENT_FWD_TO_QSERVER',
      u'19': u'GL_EVENT_SCANDLVR',
      u'20': u'GL_EVENT_BACKUP',
      u'21': u'GL_EVENT_SCAN_ABORT',
      u'22': u'GL_EVENT_RTS_LOAD_ERROR',
      u'23': u'GL_EVENT_RTS_LOAD',
      u'24': u'GL_EVENT_RTS_UNLOAD',
      u'25': u'GL_EVENT_REMOVE_CLIENT',
      u'26': u'GL_EVENT_SCAN_DELAYED',
      u'27': u'GL_EVENT_SCAN_RESTART',
      u'28': u'GL_EVENT_ADD_SAVROAMCLIENT_TOSERVER',
      u'29': u'GL_EVENT_REMOVE_SAVROAMCLIENT_FROMSERVER',
      u'30': u'GL_EVENT_LICENSE_WARNING',
      u'31': u'GL_EVENT_LICENSE_ERROR',
      u'32': u'GL_EVENT_LICENSE_GRACE',
      u'33': u'GL_EVENT_UNAUTHORIZED_COMM',
      u'34': u'GL_EVENT_LOG_FWD_THRD_ERR',
      u'35': u'GL_EVENT_LICENSE_INSTALLED',
      u'36': u'GL_EVENT_LICENSE_ALLOCATED',
      u'37': u'GL_EVENT_LICENSE_OK',
      u'38': u'GL_EVENT_LICENSE_DEALLOCATED',
      u'39': u'GL_EVENT_BAD_DEFS_ROLLBACK',
      u'40': u'GL_EVENT_BAD_DEFS_UNPROTECTED',
      u'41': u'GL_EVENT_SAV_PROVIDER_PARSING_ERROR',
      u'42': u'GL_EVENT_RTS_ERROR',
      u'43': u'GL_EVENT_COMPLIANCE_FAIL',
      u'44': u'GL_EVENT_COMPLIANCE_SUCCESS',
      u'45': u'GL_EVENT_SECURITY_SYMPROTECT_POLICYVIOLATION',
      u'46': u'GL_EVENT_ANOMALY_START',
      u'47': u'GL_EVENT_DETECTION_ACTION_TAKEN',
      u'48': u'GL_EVENT_REMEDIATION_ACTION_PENDING',
      u'49': u'GL_EVENT_REMEDIATION_ACTION_FAILED',
      u'50': u'GL_EVENT_REMEDIATION_ACTION_SUCCESSFUL',
      u'51': u'GL_EVENT_ANOMALY_FINISH',
      u'52': u'GL_EVENT_COMMS_LOGIN_FAILED',
      u'53': u'GL_EVENT_COMMS_LOGIN_SUCCESS',
      u'54': u'GL_EVENT_COMMS_UNAUTHORIZED_COMM',
      u'55': u'GL_EVENT_CLIENT_INSTALL_AV',
      u'56': u'GL_EVENT_CLIENT_INSTALL_FW',
      u'57': u'GL_EVENT_CLIENT_UNINSTALL',
      u'58': u'GL_EVENT_CLIENT_UNINSTALL_ROLLBACK',
      u'59': u'GL_EVENT_COMMS_SERVER_GROUP_ROOT_CERT_ISSUE',
      u'60': u'GL_EVENT_COMMS_SERVER_CERT_ISSUE',
      u'61': u'GL_EVENT_COMMS_TRUSTED_ROOT_CHANGE',
      u'62': u'GL_EVENT_COMMS_SERVER_CERT_STARTUP_FAILED',
      u'63': u'GL_EVENT_CLIENT_CHECKIN',
      u'64': u'GL_EVENT_CLIENT_NO_CHECKIN',
      u'65': u'GL_EVENT_SCAN_SUSPENDED',
      u'66': u'GL_EVENT_SCAN_RESUMED',
      u'67': u'GL_EVENT_SCAN_DURATION_INSUFFICIENT',
      u'68': u'GL_EVENT_CLIENT_MOVE',
      u'69': u'GL_EVENT_SCAN_FAILED_ENHANCED',
      u'70': u'GL_EVENT_MAX_event_name',
      u'71': u'GL_EVENT_HEUR_THREAT_NOW_WHITELISTED',
      u'72': u'GL_EVENT_INTERESTING_PROCESS_DETECTED_START',
      u'73': u'GL_EVENT_LOAD_ERROR_COH',
      u'74': u'GL_EVENT_LOAD_ERROR_SYKNAPPS',
      u'75': u'GL_EVENT_INTERESTING_PROCESS_DETECTED_FINISH',
      u'76': u'GL_EVENT_HPP_SCAN_NOT_SUPPORTED_FOR_OS',
      u'77': u'GL_EVENT_HEUR_THREAT_NOW_KNOWN'}

  CATEGORY_NAMES = {
      u'1': u'GL_CAT_INFECTION',
      u'2': u'GL_CAT_SUMMARY',
      u'3': u'GL_CAT_PATTERN',
      u'4': u'GL_CAT_SECURITY'}

  ACTION_1_2_NAMES = {
      u'1': u'Quarantine infected file',
      u'2': u'Rename infected file',
      u'3': u'Delete infected file',
      u'4': u'Leave alone (log only)',
      u'5': u'Clean virus from file',
      u'6': u'Clean or delete macros'}

  ACTION_0_NAMES = {
      u'1': u'Quarantined',
      u'2': u'Renamed',
      u'3': u'Deleted',
      u'4': u'Left alone',
      u'5': u'Cleaned',
      u'6': (u'Cleaned or macros deleted (no longer used as of '
             u'Symantec AntiVirus 9.x)'),
      u'7': u'Saved file as...',
      u'8': u'Sent to Intel (AMS)',
      u'9': u'Moved to backup location',
      u'10': u'Renamed backup file',
      u'11': u'Undo action in Quarantine View',
      u'12': u'Write protected or lack of permissions - Unable to act on file',
      u'13': u'Backed up file'}

  # The identifier for the formatter (a regular expression)
  FORMAT_STRING_SEPARATOR = u'; '
  FORMAT_STRING_PIECES = [
      u'Event Name: {event_map}',
      u'Category Name: {category_map}',
      u'Malware Name: {virus}',
      u'Malware Path: {file}',
      u'Action0: {action0_map}',
      u'Action1: {action1_map}',
      u'Action2: {action2_map}',
      u'Description: {description}',
      u'Scan ID: {scanid}',
      u'Event Data: {event_data}',
      u'Remote Machine: {remote_machine}',
      u'Remote IP: {remote_machine_ip}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{file}',
      u'{virus}',
      u'{action0_map}',
      u'{action1_map}',
      u'{action2_map}']

  SOURCE_LONG = u'Symantec AV Log'
  SOURCE_SHORT = u'LOG'

  def GetMessages(self, unused_formatter_mediator, event):
    """Determines the formatted message strings for an event object.

    Args:
      formatter_mediator (FormatterMediator): mediates the interactions between
          formatters and other components, such as storage and Windows EventLog
          resources.
      event (EventObject): event.

    Returns:
      tuple(str, str): formatted message string and short message string.

    Raises:
      WrongFormatter: if the event object cannot be formatted by the formatter.
    """
    if self.DATA_TYPE != event.data_type:
      raise errors.WrongFormatter(u'Unsupported data type: {0:s}.'.format(
          event.data_type))

    event_values = event.CopyToDict()

    event = event_values.get(u'event', None)
    if event:
      event_values[u'event_map'] = self.EVENT_NAMES.get(event, u'Unknown')

    category = event_values.get(u'cat', None)
    if category:
      event_values[u'category_map'] = self.CATEGORY_NAMES.get(
          category, u'Unknown')

    action = event_values.get(u'action0', None)
    if action:
      event_values[u'action0_map'] = self.ACTION_0_NAMES.get(action, u'Unknown')

    action = event_values.get(u'action1', None)
    if action:
      event_values[u'action1_map'] = self.ACTION_1_2_NAMES.get(
          action, u'Unknown')

    action = event_values.get(u'action2', None)
    if action:
      event_values[u'action2_map'] = self.ACTION_1_2_NAMES.get(
          action, u'Unknown')

    return self._ConditionalFormatMessages(event_values)


manager.FormattersManager.RegisterFormatter(SymantecAVFormatter)



# -*- coding: utf-8 -*-
"""The Docker event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class DockerBaseEventFormatter(interface.ConditionalEventFormatter):
  """Class that contains common Docker event formatter functionality."""

  DATA_TYPE = u'docker:json'

  FORMAT_STRING_SHORT_PIECES = [
      u'{id}']

  SOURCE_SHORT = u'DOCKER'


class DockerContainerLogEventFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Docker container log event"""

  DATA_TYPE = u'docker:json:container:log'

  FORMAT_STRING_SEPARATOR = u', '

  FORMAT_STRING_PIECES = (
      u'Text: {log_line}',
      u'Container ID: {container_id}',
      u'Source: {log_source}',
  )

  SOURCE_LONG = u'Docker Container Logs'
  SOURCE_SHORT = u'DOCKER'


class DockerLayerEventFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Docker layer event."""

  DATA_TYPE = u'docker:json:layer'

  FORMAT_STRING_SEPARATOR = u', '

  FORMAT_STRING_PIECES = (
      u'Command: {command}',
      u'Layer ID: {layer_id}',
  )

  SOURCE_LONG = u'Docker Layer'
  SOURCE_SHORT = u'DOCKER'


class DockerContainerEventFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Docker event."""

  DATA_TYPE = u'docker:json:container'

  FORMAT_STRING_SEPARATOR = u', '

  FORMAT_STRING_PIECES = [
      u'Action: {action}',
      u'Container Name: {container_name}',
      u'Container ID: {container_id}',
  ]

  SOURCE_LONG = u'Docker Container'
  SOURCE_SHORT = u'DOCKER'


manager.FormattersManager.RegisterFormatters([
    DockerContainerEventFormatter,
    DockerContainerLogEventFormatter,
    DockerLayerEventFormatter,
])



# -*- coding: utf-8 -*-
"""Artifact attribute containers."""

from plaso.containers import interface
from plaso.containers import manager


class ArtifactAttributeContainer(interface.AttributeContainer):
  """Base class to represent an artifact attribute container."""


class EnvironmentVariableArtifact(ArtifactAttributeContainer):
  """Environment variable artifact attribute container.

  Also see:
    https://en.wikipedia.org/wiki/Environment_variable

  Attributes:
    case_sensitive (bool): True if environment variable name is case sensitive.
    name (str): environment variable name e.g. 'SystemRoot' as in
        '%SystemRoot%' or 'HOME' in '$HOME'.
    value (str): environment variable value e.g. 'C:\\Windows' or '/home/user'.
  """
  CONTAINER_TYPE = u'environment_variable'

  def __init__(self, case_sensitive=True, name=None, value=None):
    """Initializes an environment variable artifact.

    Args:
      case_sensitive (Optional[bool]): True if environment variable name
          is case sensitive.
      name (Optional[str]): environment variable name.
      value (Optional[str]): environment variable value.
    """
    super(EnvironmentVariableArtifact, self).__init__()
    self.case_sensitive = case_sensitive
    self.name = name
    self.value = value


class HostnameArtifact(ArtifactAttributeContainer):
  """Hostname artifact attribute container.

  Also see:
    https://en.wikipedia.org/wiki/Hostname
    http://cybox.mitre.org/language/version2.1/xsddocs/objects/
    Hostname_Object.html

  Attributes:
    name (str): name of the host according to the naming schema.
    schema (str): naming schema e.g. DNS, NIS, SMB/NetBIOS.
  """
  CONTAINER_TYPE = u'hostname'

  def __init__(self, name=None, schema=u'DNS'):
    """Initializes a hostname artifact.

    Args:
      name (Optional[str]): name of the host according to the naming schema.
      schema (Optional[str]): naming schema.
    """
    super(HostnameArtifact, self).__init__()
    self.name = name
    self.schema = schema


class SystemConfigurationArtifact(ArtifactAttributeContainer):
  """System configuration artifact attribute container.

  The system configuration contains the configuration data of a specific
  system installation e.g. Windows or Linux.

  Attributes:
    code_page (str): system code page.
    hostname (HostnameArtifact): hostname.
    keyboard_layout (str): keyboard layout.
    operating_system (str): operating system for example "Mac OS X" or
        "Windows".
    operating_system_product (str): operating system product for example
        "Windows XP".
    operating_system_version (str): operating system version for example
        "10.9.2" or "8.1".
    time_zone (str): system time zone.
    user_accounts (list[UserAccountArtifact]): user accounts.
  """
  CONTAINER_TYPE = u'system_configuration'

  def __init__(self, code_page=None, time_zone=None):
    """Initializes a system configuration artifact.

    Args:
      code page (Optional[str]): system code page.
      time_zone (Optional[str]): system time zone.
    """
    super(SystemConfigurationArtifact, self).__init__()
    self.code_page = code_page
    self.hostname = None
    self.keyboard_layout = None
    self.operating_system = None
    self.operating_system_product = None
    self.operating_system_version = None
    self.time_zone = time_zone
    self.user_accounts = []


class UserAccountArtifact(ArtifactAttributeContainer):
  """User account artifact attribute container.

  Also see:
    http://cybox.mitre.org/language/version2.1/xsddocs/objects/
    User_Account_Object.html

  Attributes:
    full_name (str): name describing the user e.g. full name.
    group_identifier (str): identifier of the primary group the user is part of.
    identifier (str): user identifier.
    user_directory (str): path of the user (or home or profile) directory.
    username (str): name uniquely identifying the user.
  """
  CONTAINER_TYPE = u'user_account'

  def __init__(
      self, full_name=None, group_identifier=None, identifier=None,
      user_directory=None, username=None):
    """Initializes an user artifact.

    Args:
      full_name (Optional[str]): name describing the user e.g. full name.
      group_identifier (Optional[str]): identifier of the primary group
          the user is part of.
      identifier (Optional[str]): user identifier.
      user_directory (Optional[str]): path of the user (or home or profile)
          directory.
      username (Optional[str]): name uniquely identifying the user.
    """
    super(UserAccountArtifact, self).__init__()
    self.full_name = full_name
    self.group_identifier = group_identifier
    self.identifier = identifier
    # TODO: add shell.
    self.user_directory = user_directory
    self.username = username


manager.AttributeContainersManager.RegisterAttributeContainers([
    EnvironmentVariableArtifact, HostnameArtifact, SystemConfigurationArtifact,
    UserAccountArtifact])



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the VirusTotal analysis plugin."""

import unittest

try:
  import mock  # pylint: disable=import-error
except ImportError:
  from unittest import mock

from dfdatetime import posix_time as dfdatetime_posix_time
from dfvfs.path import fake_path_spec

from plaso.analysis import virustotal
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import timelib

from tests.analysis import test_lib


class MockResponse(dict):
  """A mock object to simulate a response object from the requests library."""

  def json(self):
    """Provided for compatibility with the requests library."""
    return self

  def raise_for_status(self):
    """Provided for compatibility with the requests library."""
    return


class VirusTotalTest(test_lib.AnalysisPluginTestCase):
  """Tests for the VirusTotal analysis plugin."""

  _EVENT_1_HASH = u'90'

  _FAKE_API_KEY = u'4'

  _TEST_EVENTS = [{
      u'timestamp': timelib.Timestamp.CopyFromString(u'2015-01-01 17:00:00'),
      u'sha256_hash': _EVENT_1_HASH}]

  def _MockGet(self, url, params):
    """Mock function to simulate a VirusTotal API request.

    Args:
      url (str): URL being requested.
      params (dict[str, object]): HTTP parameters for the VirusTotal API
          request.

    Returns:
      MockResponse: mocked response that simulates a real response object
          returned by the requests library from the VirusTotal API.
    """
    # pylint: disable=protected-access
    self.assertEqual(
        url, virustotal.VirusTotalAnalyzer._VIRUSTOTAL_API_REPORT_URL)
    if params[u'resource'] == self._EVENT_1_HASH:
      response = MockResponse()
      response[u'resource'] = self._EVENT_1_HASH
      response[u'response_code'] = 1
      response[u'positives'] = 10
      return response
    self.fail(u'Unexpected parameters to request.get()')

  def _CreateTestEventObject(self, event_dictionary):
    """Create a test event with a set of attributes.

    Args:
      event_dictionary (dict[str, str]): contains attributes of an event to add
          to the queue.

    Returns:
      EventObject: event with the appropriate attributes for testing.
    """
    date_time = dfdatetime_posix_time.PosixTime(
        timestamp=event_dictionary[u'timestamp'])
    event = time_events.DateTimeValuesEvent(
        date_time, definitions.TIME_DESCRIPTION_CREATION)

    event.data_type = u'pe:compilation:compilation_time'
    event.pe_type = u'Executable (EXE)'

    for attribute_name, attribute_value in event_dictionary.items():
      if attribute_name == u'timestamp':
        continue

      setattr(event, attribute_name, attribute_value)

    return event

  def setUp(self):
    """Makes preparations before running an individual test."""
    self.requests_patcher = mock.patch(u'requests.get', self._MockGet)
    self.requests_patcher.start()

  def tearDown(self):
    """Cleans up after running an individual test."""
    self.requests_patcher.stop()

  def testExamineEventAndCompileReport(self):
    """Tests the ExamineEvent and CompileReport functions."""
    events = []
    for event_dictionary in self._TEST_EVENTS:
      event_dictionary[u'pathspec'] = fake_path_spec.FakePathSpec(
          location=u'C:\\WINDOWS\\system32\\evil.exe')

      event = self._CreateTestEventObject(event_dictionary)
      events.append(event)

    plugin = virustotal.VirusTotalAnalysisPlugin()
    plugin.SetAPIKey(self._FAKE_API_KEY)

    storage_writer = self._AnalyzeEvents(events, plugin)

    self.assertEqual(len(storage_writer.analysis_reports), 1)
    self.assertEqual(storage_writer.number_of_event_tags, 1)

    report = storage_writer.analysis_reports[0]
    self.assertIsNotNone(report)

    expected_text = (
        u'virustotal hash tagging results\n'
        u'1 path specifications tagged with label: virustotal_detections_10\n')
    self.assertEqual(report.text, expected_text)

    labels = []
    for event_tag in storage_writer.GetEventTags():
      labels.extend(event_tag.labels)
    self.assertEqual(len(labels), 1)

    expected_labels = [u'virustotal_detections_10']
    self.assertEqual(labels, expected_labels)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The plist file object."""

import binascii
import os
import plistlib

from binplist import binplist


class PlistFile(object):
  """Class that defines a plist file.

  Attributes:
    root_key: the plist root key (instance of plistlib._InternalDict).
  """

  def __init__(self):
    """Initializes the plist file object."""
    super(PlistFile, self).__init__()
    self.root_key = None

  def GetValueByPath(self, path_segments):
    """Retrieves a plist value by path.

    Args:
      path_segments: a list of path segments strings relative from the root
                     of the plist.

    Returns:
      The value of the key specified by the path or None.
    """
    key = self.root_key
    for path_segment in path_segments:
      # pylint: disable=protected-access
      if isinstance(key, (dict, plistlib._InternalDict)):
        key = key.get(path_segment)

      elif isinstance(key, list):
        try:
          list_index = int(path_segment, 10)
        except ValueError:
          return

        key = key[list_index]

      else:
        return

      if not key:
        return

    return key

  def Read(self, file_object):
    """Reads a plist from a file-like object.

    Args:
      file_object: the file-like object.

    Raises:
      IOError: if the plist file-like object cannot be read.
    """
    try:
      file_object.seek(0, os.SEEK_SET)
      # Note that binplist.readPlist does not seek to offset 0.
      self.root_key = binplist.readPlist(file_object)

    except (
        AttributeError, binascii.Error, binplist.FormatError, LookupError,
        OverflowError, ValueError) as exception:
      raise IOError(exception)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Mac OS X appfirewall.log file event formatter."""

import unittest

from plaso.formatters import mac_document_versions

from tests.formatters import test_lib


class MacDocumentVersionsFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the Mac OS X Document Versions page visited event formatter."""

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = mac_document_versions.MacDocumentVersionsFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = mac_document_versions.MacDocumentVersionsFormatter()

    expected_attribute_names = [
        u'name',
        u'path',
        u'version_path',
        u'user_sid']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  # TODO: add test for GetMessages.


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Mac OS X wifi.log file event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class MacWifiLogFormatter(interface.ConditionalEventFormatter):
  """Formatter for a wifi.log file event."""

  DATA_TYPE = u'mac:wifilog:line'

  FORMAT_STRING_PIECES = [
      u'Action: {action}',
      u'Agent: {agent}',
      u'({function})',
      u'Log: {text}']

  FORMAT_STRING_SHORT_PIECES = [
      u'Action: {action}']

  SOURCE_LONG = u'Mac Wifi Log'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatter(MacWifiLogFormatter)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the Less Frequently Used (LFU) Windows Registry plugin."""

import unittest

from dfdatetime import filetime as dfdatetime_filetime
from dfwinreg import definitions as dfwinreg_definitions
from dfwinreg import fake as dfwinreg_fake

from plaso.formatters import winreg  # pylint: disable=unused-import
from plaso.lib import timelib
from plaso.parsers.winreg_plugins import lfu

from tests.parsers.winreg_plugins import test_lib


class TestBootExecutePlugin(test_lib.RegistryPluginTestCase):
  """Tests for the LFU BootExecute Windows Registry plugin."""

  def _CreateTestKey(self, key_path, time_string):
    """Creates Registry keys and values for testing.

    Args:
      key_path (str): Windows Registry key path.
      time_string (str): key last written date and time.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromString(time_string)
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        u'Session Manager', key_path=key_path,
        last_written_time=filetime.timestamp, offset=153)

    value_data = u'autocheck autochk *\x00'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'BootExecute', data=value_data,
        data_type=dfwinreg_definitions.REG_MULTI_SZ, offset=123)
    registry_key.AddValue(registry_value)

    value_data = u'2592000'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'CriticalSectionTimeout', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ, offset=153)
    registry_key.AddValue(registry_value)

    value_data = u'\x00'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ExcludeFromKnownDlls', data=value_data,
        data_type=dfwinreg_definitions.REG_MULTI_SZ, offset=163)
    registry_key.AddValue(registry_value)

    value_data = u'0'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'GlobalFlag', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=173)
    registry_key.AddValue(registry_value)

    value_data = u'0'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'HeapDeCommitFreeBlockThreshold', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ, offset=183)
    registry_key.AddValue(registry_value)

    value_data = u'0'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'HeapDeCommitTotalFreeThreshold', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ, offset=203)
    registry_key.AddValue(registry_value)

    value_data = u'0'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'HeapSegmentCommit', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ, offset=213)
    registry_key.AddValue(registry_value)

    value_data = u'0'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'HeapSegmentReserve', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ, offset=223)
    registry_key.AddValue(registry_value)

    value_data = u'2'.encode(u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'NumberOfInitialSessions', data=value_data,
        data_type=dfwinreg_definitions.REG_SZ, offset=243)
    registry_key.AddValue(registry_value)

    return registry_key

  def testProcess(self):
    """Tests the Process function."""
    key_path = (
        u'HKEY_LOCAL_MACHINE\\System\\ControlSet001\\Control\\Session Manager')
    time_string = u'2012-08-31 20:45:29'
    registry_key = self._CreateTestKey(key_path, time_string)

    plugin = lfu.BootExecutePlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    self.assertEqual(storage_writer.number_of_events, 2)

    events = list(storage_writer.GetEvents())

    event = events[0]

    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(time_string)
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = (
        u'[{0:s}] BootExecute: autocheck autochk *').format(key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)

    event = events[1]

    expected_message = (
        u'[{0:s}] '
        u'CriticalSectionTimeout: 2592000 '
        u'ExcludeFromKnownDlls: [] '
        u'GlobalFlag: 0 '
        u'HeapDeCommitFreeBlockThreshold: 0 '
        u'HeapDeCommitTotalFreeThreshold: 0 '
        u'HeapSegmentCommit: 0 '
        u'HeapSegmentReserve: 0 '
        u'NumberOfInitialSessions: 2').format(key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


class TestBootVerificationRegistry(test_lib.RegistryPluginTestCase):
  """Tests for the LFU BootVerification Windows Registry plugin."""

  def _CreateTestKey(self, key_path, time_string):
    """Creates Registry keys and values for testing.

    Args:
      key_path (str): Windows Registry key path.
      time_string (str): key last written date and time.

    Returns:
      dfwinreg.WinRegistryKey: a Windows Registry key.
    """
    filetime = dfdatetime_filetime.Filetime()
    filetime.CopyFromString(time_string)
    registry_key = dfwinreg_fake.FakeWinRegistryKey(
        u'BootVerificationProgram', key_path=key_path,
        last_written_time=filetime.timestamp, offset=153)

    value_data = u'C:\\WINDOWS\\system32\\googleupdater.exe'.encode(
        u'utf_16_le')
    registry_value = dfwinreg_fake.FakeWinRegistryValue(
        u'ImagePath', data=value_data, data_type=dfwinreg_definitions.REG_SZ,
        offset=123)
    registry_key.AddValue(registry_value)

    return registry_key

  def testProcess(self):
    """Tests the Process function."""
    key_path = u'\\ControlSet001\\Control\\BootVerificationProgram'
    time_string = u'2012-08-31 20:45:29'
    registry_key = self._CreateTestKey(key_path, time_string)

    plugin = lfu.BootVerificationPlugin()
    storage_writer = self._ParseKeyWithPlugin(registry_key, plugin)

    self.assertEqual(storage_writer.number_of_events, 1)

    events = list(storage_writer.GetEvents())

    event = events[0]

    # This should just be the plugin name, as we're invoking it directly,
    # and not through the parser.
    self.assertEqual(event.parser, plugin.plugin_name)

    expected_timestamp = timelib.Timestamp.CopyFromString(time_string)
    self.assertEqual(event.timestamp, expected_timestamp)

    expected_message = (
        u'[{0:s}] '
        u'ImagePath: C:\\WINDOWS\\system32\\googleupdater.exe').format(
            key_path)
    expected_short_message = u'{0:s}...'.format(expected_message[:77])

    self._TestGetMessageStrings(event, expected_message, expected_short_message)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Skype main database event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class SkypeAccountFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Skype account event."""

  DATA_TYPE = u'skype:event:account'

  FORMAT_STRING_PIECES = [
      u'{username}',
      u'[{email}]',
      u'Country: {country}']

  SOURCE_LONG = u'Skype Account'
  SOURCE_SHORT = u'LOG'


class SkypeChatFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Skype chat message event."""

  DATA_TYPE = u'skype:event:chat'

  FORMAT_STRING_PIECES = [
      u'From: {from_account}',
      u'To: {to_account}',
      u'[{title}]',
      u'Message: [{text}]']

  FORMAT_STRING_SHORT_PIECES = [
      u'From: {from_account}',
      u'To: {to_account}']

  SOURCE_LONG = u'Skype Chat MSG'
  SOURCE_SHORT = u'LOG'


class SkypeSMSFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Skype SMS event."""

  DATA_TYPE = u'skype:event:sms'

  FORMAT_STRING_PIECES = [
      u'To: {number}',
      u'[{text}]']

  SOURCE_LONG = u'Skype SMS'
  SOURCE_SHORT = u'LOG'


class SkypeCallFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Skype call event."""

  DATA_TYPE = u'skype:event:call'

  FORMAT_STRING_PIECES = [
      u'From: {src_call}',
      u'To: {dst_call}',
      u'[{call_type}]']

  SOURCE_LONG = u'Skype Call'
  SOURCE_SHORT = u'LOG'


class SkypeTransferFileFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Skype transfer file event."""

  DATA_TYPE = u'skype:event:transferfile'

  FORMAT_STRING_PIECES = [
      u'Source: {source}',
      u'Destination: {destination}',
      u'File: {transferred_filename}',
      u'[{action_type}]']

  SOURCE_LONG = u'Skype Transfer Files'
  SOURCE_SHORT = u'LOG'


manager.FormattersManager.RegisterFormatters([
    SkypeAccountFormatter, SkypeChatFormatter, SkypeSMSFormatter,
    SkypeCallFormatter, SkypeTransferFileFormatter])



# -*- coding: utf-8 -*-
"""Parser for Windows Shortcut (LNK) files."""

import uuid

import pylnk

from dfdatetime import filetime as dfdatetime_filetime
from dfdatetime import semantic_time as dfdatetime_semantic_time
from dfdatetime import uuid_time as dfdatetime_uuid_time

from plaso.containers import time_events
from plaso.containers import windows_events
from plaso.lib import definitions
from plaso.lib import specification
from plaso.parsers import interface
from plaso.parsers import manager
from plaso.parsers.shared import shell_items


class WinLnkLinkEvent(time_events.DateTimeValuesEvent):
  """Convenience class for a Windows Shortcut (LNK) link event.

  Attributes:
    birth_droid_file_identifier (str): distributed link tracking brith droid
        file identifier.
    birth_droid_volume_identifier (str): distributed link tracking brith droid
        volume identifier.
    command_line_arguments (str): command line arguments.
    description (str): description of the linked item.
    drive_serial_number (int): drive serial number where the linked item
        resides.
    drive_type (str): drive type where the linked item resided.
    droid_file_identifier (str): distributed link tracking droid file
        identifier.
    droid_volume_identifier (str): distributed link tracking droid volume
        identifier.
    env_var_location (str): evironment variables loction.
    file_attribute_flags (int): file attribute flags of the linked item.
    file_size (int): size of the linked item.
    icon_location (str): icon location.
    link_target (str): shell item list of the link target.
    local_path (str): local path of the linked item.
    network_path (str): local path of the linked item.
    offset (int): data offset of the event.
    relative_path (str): relative path.
    volume_label (str): volume label where the linked item resided.
    working_directory (str): working directory.
  """

  DATA_TYPE = u'windows:lnk:link'

  def __init__(
      self, date_time, date_time_description, lnk_file, link_target, offset=0):
    """Initializes an event.

    Args:
      date_time (dfdatetime.DateTimeValues): date and time values.
      date_time_description (str): description of the meaning of the date
          and time values.
      lnk_file (pylnk.file): LNK file.
      link_target (str): link target shell item list.
      offset (Optional[int]): data offset of the event.
    """
    super(WinLnkLinkEvent, self).__init__(date_time, date_time_description)
    self.birth_droid_file_identifier = lnk_file.birth_droid_file_identifier
    self.birth_droid_volume_identifier = lnk_file.birth_droid_volume_identifier
    self.command_line_arguments = lnk_file.command_line_arguments
    self.description = lnk_file.description
    self.drive_serial_number = lnk_file.drive_serial_number
    self.drive_type = lnk_file.drive_type
    self.droid_file_identifier = lnk_file.droid_file_identifier
    self.droid_volume_identifier = lnk_file.droid_volume_identifier
    self.env_var_location = lnk_file.environment_variables_location
    self.file_attribute_flags = lnk_file.file_attribute_flags
    self.file_size = lnk_file.file_size
    self.icon_location = lnk_file.icon_location
    self.link_target = link_target
    self.local_path = lnk_file.local_path
    self.network_path = lnk_file.network_path
    self.offset = offset
    self.relative_path = lnk_file.relative_path
    self.volume_label = lnk_file.volume_label
    self.working_directory = lnk_file.working_directory


class WinLnkParser(interface.FileObjectParser):
  """Parses Windows Shortcut (LNK) files."""

  _INITIAL_FILE_OFFSET = None

  NAME = u'lnk'
  DESCRIPTION = u'Parser for Windows Shortcut (LNK) files.'

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification.

    Returns:
      FormatSpecification: format specification.
    """
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(
        b'\x01\x14\x02\x00\x00\x00\x00\x00\xc0\x00\x00\x00\x00\x00\x00\x46',
        offset=4)
    return format_specification

  def _ParseDistributedTrackingIdentifier(
      self, parser_mediator, uuid_string, origin):
    """Extracts data from a Distributed Tracking identifier.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      uuid_string (str): UUID string of the Distributed Tracking identifier.
      origin (str): origin of the event (event source).
    """
    uuid_object = uuid.UUID(uuid_string)

    if uuid_object.version == 1:
      event_data = windows_events.WindowsDistributedLinkTrackingEventData(
          uuid_object, origin)
      date_time = dfdatetime_uuid_time.UUIDTime(timestamp=uuid_object.time)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_CREATION)
      parser_mediator.ProduceEventWithEventData(event, event_data)

  def ParseFileObject(
      self, parser_mediator, file_object, display_name=None, **kwargs):
    """Parses a Windows Shortcut (LNK) file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): file-like object.
      display_name (Optional[str]): display name.
    """
    if not display_name:
      display_name = parser_mediator.GetDisplayName()

    lnk_file = pylnk.file()
    lnk_file.set_ascii_codepage(parser_mediator.codepage)

    try:
      lnk_file.open_file_object(file_object)
    except IOError as exception:
      parser_mediator.ProduceExtractionError(
          u'unable to open file with error: {0:s}'.format(exception))
      return

    link_target = None
    if lnk_file.link_target_identifier_data:
      # TODO: change file_entry.name to display name once it is generated
      # correctly.
      display_name = parser_mediator.GetFilename()
      shell_items_parser = shell_items.ShellItemsParser(display_name)
      shell_items_parser.ParseByteStream(
          parser_mediator, lnk_file.link_target_identifier_data,
          codepage=parser_mediator.codepage)

      link_target = shell_items_parser.CopyToPath()

    access_time = lnk_file.get_file_access_time_as_integer()
    if access_time != 0:
      date_time = dfdatetime_filetime.Filetime(timestamp=access_time)
      event = WinLnkLinkEvent(
          date_time, definitions.TIME_DESCRIPTION_LAST_ACCESS, lnk_file,
          link_target)
      parser_mediator.ProduceEvent(event)

    creation_time = lnk_file.get_file_creation_time_as_integer()
    if creation_time != 0:
      date_time = dfdatetime_filetime.Filetime(timestamp=creation_time)
      event = WinLnkLinkEvent(
          date_time, definitions.TIME_DESCRIPTION_CREATION, lnk_file,
          link_target)
      parser_mediator.ProduceEvent(event)

    modification_time = lnk_file.get_file_modification_time_as_integer()
    if modification_time != 0:
      date_time = dfdatetime_filetime.Filetime(timestamp=modification_time)
      event = WinLnkLinkEvent(
          date_time, definitions.TIME_DESCRIPTION_MODIFICATION,
          lnk_file, link_target)
      parser_mediator.ProduceEvent(event)

    if access_time == 0 and creation_time == 0 and modification_time == 0:
      date_time = dfdatetime_semantic_time.SemanticTime(u'Not set')
      event = WinLnkLinkEvent(
          date_time, definitions.TIME_DESCRIPTION_NOT_A_TIME, lnk_file,
          link_target)
      parser_mediator.ProduceEvent(event)

    if lnk_file.droid_file_identifier:
      try:
        self._ParseDistributedTrackingIdentifier(
            parser_mediator, lnk_file.droid_file_identifier, display_name)
      except (TypeError, ValueError) as exception:
        parser_mediator.ProduceExtractionError(
            u'unable to read droid file identifier with error: {0:s}.'.format(
                exception))

    if lnk_file.birth_droid_file_identifier:
      try:
        self._ParseDistributedTrackingIdentifier(
            parser_mediator, lnk_file.birth_droid_file_identifier, display_name)
      except (TypeError, ValueError) as exception:
        parser_mediator.ProduceExtractionError((
            u'unable to read birth droid file identifier with error: '
            u'{0:s}.').format(exception))

    lnk_file.close()


manager.ParsersManager.RegisterParser(WinLnkParser)



# -*- coding: utf-8 -*-
"""This file contains constants for making service keys more readable."""

SERVICE_ENUMS = {
    # Human readable strings for the service type.
    u'Type': {
        1: u'Kernel Device Driver (0x1)',
        2: u'File System Driver (0x2)',
        4: u'Adapter (0x4)',
        16: u'Service - Own Process (0x10)',
        32: u'Service - Share Process (0x20)'
    },
    # Human readable strings for the service start type.
    u'Start': {
        0: u'Boot (0)',
        1: u'System (1)',
        2: u'Auto Start (2)',
        3: u'Manual (3)',
        4: u'Disabled (4)'
    },
    # Human readable strings for the error handling.
    u'ErrorControl': {
        0: u'Ignore (0)',
        1: u'Normal (1)',
        2: u'Severe (2)',
        3: u'Critical (3)'
    }
}



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the event filters manager."""

import unittest

from plaso.filters import manager

from tests.filters import test_lib


class FiltersManagerTest(unittest.TestCase):
  """Tests for the event filters manager."""

  def testFilterRegistration(self):
    """Tests the RegisterFilter and DeregisterFilter functions."""
    # pylint: disable=protected-access
    number_of_filters = len(manager.FiltersManager._filter_classes)

    manager.FiltersManager.RegisterFilter(test_lib.TestEventFilter)
    self.assertEqual(
        len(manager.FiltersManager._filter_classes),
        number_of_filters + 1)

    with self.assertRaises(KeyError):
      manager.FiltersManager.RegisterFilter(test_lib.TestEventFilter)

    manager.FiltersManager.DeregisterFilter(test_lib.TestEventFilter)
    self.assertEqual(
        len(manager.FiltersManager._filter_classes),
        number_of_filters)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The Google Chrome cookies database event formatter."""

from plaso.formatters import interface
from plaso.formatters import manager


class ChromeCookieFormatter(interface.ConditionalEventFormatter):
  """Formatter for a Chrome cookie event."""

  DATA_TYPE = u'chrome:cookie:entry'

  FORMAT_STRING_PIECES = [
      u'{url}',
      u'({cookie_name})',
      u'Flags:',
      u'[HTTP only] = {httponly}',
      u'[Persistent] = {persistent}']

  FORMAT_STRING_SHORT_PIECES = [
      u'{host}',
      u'({cookie_name})']

  SOURCE_LONG = u'Chrome Cookies'
  SOURCE_SHORT = u'WEBHIST'


manager.FormattersManager.RegisterFormatter(ChromeCookieFormatter)



# -*- coding: utf-8 -*-
"""The cookie plugins manager object."""


class CookiePluginsManager(object):
  """Class that implements the cookie plugins manager."""

  _plugin_classes = {}

  @classmethod
  def DeregisterPlugin(cls, plugin_class):
    """Deregisters a plugin class.

    The plugin classes are identified based on their lower case name.

    Args:
      plugin_class: the class object of the plugin.

    Raises:
      KeyError: if plugin class is not set for the corresponding name.
    """
    plugin_name = plugin_class.NAME.lower()
    if plugin_name not in cls._plugin_classes:
      raise KeyError(u'Plugin class not set for name: {0:s}.'.format(
          plugin_class.NAME))

    del cls._plugin_classes[plugin_name]

  @classmethod
  def GetPlugins(cls):
    """Retrieves the cookie plugins.

    Returns:
      A list of all cookie plugin objects.
    """
    return [plugin_class() for plugin_class in iter(
        cls._plugin_classes.values())]

  @classmethod
  def RegisterPlugin(cls, plugin_class):
    """Registers a plugin class.

    The plugin classes are identified based on their lower case name.

    Args:
      plugin_class: the class object of the plugin.

    Raises:
      KeyError: if plugin class is already set for the corresponding name.
    """
    plugin_name = plugin_class.NAME.lower()
    if plugin_name in cls._plugin_classes:
      raise KeyError((u'Plugin class already set for name: {0:s}.').format(
          plugin_class.NAME))

    cls._plugin_classes[plugin_name] = plugin_class

  @classmethod
  def RegisterPlugins(cls, plugin_classes):
    """Registers plugin classes.

    The plugin classes are identified based on their lower case name.

    Args:
      plugin_classes: a list of class objects of the plugins.

    Raises:
      KeyError: if plugin class is already set for the corresponding name.
    """
    for plugin_class in plugin_classes:
      cls.RegisterPlugin(plugin_class)



# -*- coding: utf-8 -*-
"""Spotlight searched terms plist plugin."""

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.containers import plist_event
from plaso.containers import time_events
from plaso.lib import definitions
from plaso.lib import timelib
from plaso.parsers import plist
from plaso.parsers.plist_plugins import interface


__author__ = 'Joaquin Moreno Garijo (Joaquin.MorenoGarijo.2013@live.rhul.ac.uk)'


class SpotlightPlugin(interface.PlistPlugin):
  """Basic plugin to extract information from Spotlight plist file.

  Further information about extracted fields:
    name of the item:
      search term.

    PATH:
      path of the program associated to the term.

    LAST_USED:
      last time when it was executed.

    DISPLAY_NAME:
      the display name of the program associated.
  """

  NAME = u'spotlight'
  DESCRIPTION = u'Parser for Spotlight plist files.'

  PLIST_PATH = u'com.apple.spotlight.plist'
  PLIST_KEYS = frozenset([u'UserShortcuts'])

  def GetEntries(self, parser_mediator, match=None, **unused_kwargs):
    """Extracts relevant Spotlight entries.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      match (Optional[dict[str: object]]): keys extracted from PLIST_KEYS.
    """
    shortcuts = match.get(u'UserShortcuts', {})
    for search_text, data in iter(shortcuts.items()):
      datetime_value = data.get(u'LAST_USED', None)
      if not datetime_value:
        continue

      display_name = data.get(u'DISPLAY_NAME', u'<DISPLAY_NAME>')
      path = data.get(u'PATH', u'<PATH>')

      event_data = plist_event.PlistTimeEventData()
      event_data.desc = (
          u'Spotlight term searched "{0:s}" associate to {1:s} ({2:s})').format(
              search_text, display_name, path)
      event_data.key = search_text
      event_data.root = u'/UserShortcuts'

      timestamp = timelib.Timestamp.FromPythonDatetime(datetime_value)
      date_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
          timestamp=timestamp)
      event = time_events.DateTimeValuesEvent(
          date_time, definitions.TIME_DESCRIPTION_WRITTEN)
      parser_mediator.ProduceEventWithEventData(event, event_data)


plist.PlistParser.RegisterPlugin(SpotlightPlugin)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Tests for the filters."""

import unittest

from plaso.containers import events
from plaso.formatters import interface as formatters_interface
from plaso.formatters import manager as formatters_manager
from plaso.lib import errors
from plaso.lib import pfilter
from plaso.lib import timelib


class PfilterFakeFormatter(formatters_interface.EventFormatter):
  """A formatter for this fake class."""
  DATA_TYPE = u'Weirdo:Made up Source:Last Written'

  FORMAT_STRING = u'{text}'
  FORMAT_STRING_SHORT = u'{text_short}'

  SOURCE_LONG = u'Fake Parsing Source'
  SOURCE_SHORT = u'REG'


formatters_manager.FormattersManager.RegisterFormatter(PfilterFakeFormatter)


class PFilterTest(unittest.TestCase):
  """Simple plaso specific tests to the pfilter implementation."""

  def _RunPlasoTest(self, event, query, result):
    """Run a simple test against an event object."""
    my_parser = pfilter.BaseParser(query).Parse()
    matcher = my_parser.Compile(
        pfilter.PlasoAttributeFilterImplementation)

    self.assertEqual(result, matcher.Matches(event))

  def testPlasoEvents(self):
    """Test plaso EventObjects, both Python and Protobuf version.

    These are more plaso specific tests than the more generic object filter
    ones. It will create an event object that stores some attributes. These
    objects will then be serialzed and all tests run against both the Python
    objects as well as the serialized ones.
    """
    event = events.EventObject()
    event.data_type = u'Weirdo:Made up Source:Last Written'
    event.timestamp = timelib.Timestamp.CopyFromString(
        u'2015-11-18 01:15:43')
    event.timestamp_desc = u'Last Written'
    event.text_short = (
        u'This description is different than the long one.')
    event.text = (
        u'User did a very bad thing, bad, bad thing that awoke Dr. Evil.')
    event.filename = (
        u'/My Documents/goodfella/Documents/Hideout/myfile.txt')
    event.hostname = u'Agrabah'
    event.parser = u'Weirdo'
    event.inode = 1245
    event.mydict = {
        u'value': 134, u'another': u'value', u'A Key (with stuff)': u'Here'}
    event.display_name = u'unknown:{0:s}'.format(event.filename)

    event.tag = events.EventTag(comment=u'comment')
    event.tag.AddLabel(u'browser_search')

    # Series of tests.
    query = u'filename contains \'GoodFella\''
    self._RunPlasoTest(event, query, True)

    # Double negative matching -> should be the same
    # as a positive one.
    query = u'filename not not contains \'GoodFella\''
    my_parser = pfilter.BaseParser(query)
    self.assertRaises(errors.ParseError, my_parser.Parse)

    # Test date filtering.
    query = u'date >= \'2015-11-18\''
    self._RunPlasoTest(event, query, True)

    query = u'date < \'2015-11-19\''
    self._RunPlasoTest(event, query, True)

    # 2015-11-18T01:15:43
    query = (
        u'date < \'2015-11-18T01:15:44.341\' and '
        u'date > \'2015-11-18 01:15:42\'')
    self._RunPlasoTest(event, query, True)

    query = u'date > \'2015-11-19\''
    self._RunPlasoTest(event, query, False)

    # Perform few attribute tests.
    query = u'filename not contains \'sometext\''
    self._RunPlasoTest(event, query, True)

    query = (
        u'timestamp_desc CONTAINS \'written\' AND date > \'2015-11-18\' AND '
        u'date < \'2015-11-25 12:56:21\' AND (source_short contains \'LOG\' or '
        u'source_short CONTAINS \'REG\')')
    self._RunPlasoTest(event, query, True)

    query = u'parser is not \'Made\''
    self._RunPlasoTest(event, query, True)

    query = u'parser is not \'Weirdo\''
    self._RunPlasoTest(event, query, False)

    query = u'mydict.value is 123'
    self._RunPlasoTest(event, query, False)

    query = u'mydict.akeywithstuff contains "ere"'
    self._RunPlasoTest(event, query, True)

    query = u'mydict.value is 134'
    self._RunPlasoTest(event, query, True)

    query = u'mydict.value < 200'
    self._RunPlasoTest(event, query, True)

    query = u'mydict.another contains "val"'
    self._RunPlasoTest(event, query, True)

    query = u'mydict.notthere is 123'
    self._RunPlasoTest(event, query, False)

    query = u'source_long not contains \'Fake\''
    self._RunPlasoTest(event, query, False)

    query = u'source is \'REG\''
    self._RunPlasoTest(event, query, True)

    query = u'source is not \'FILE\''
    self._RunPlasoTest(event, query, True)

    query = u'tag contains \'browser_search\''
    self._RunPlasoTest(event, query, True)

    # Multiple attributes.
    query = (
        u'source_long is \'Fake Parsing Source\' AND description_long '
        u'regexp \'bad, bad thing [\\sa-zA-Z\\.]+ evil\'')
    self._RunPlasoTest(event, query, False)

    query = (
        u'source_long is \'Fake Parsing Source\' AND text iregexp '
        u'\'bad, bad thing [\\sa-zA-Z\\.]+ evil\'')
    self._RunPlasoTest(event, query, True)


if __name__ == "__main__":
  unittest.main()
